#include "dbsim.h"
#include "object.h"
#include "language.h"
#include <string.h>
#include <ctype.h>

///////////////////////////////////////////////////////////////////////////////
//
//  Name:        SetFlags
//
//  Function:    Parses command line flags and records them in global vars.
//
//  Change Activity:
//
//     08/xx/91 (KPB) - Initial version.
//
///////////////////////////////////////////////////////////////////////////////

// Global vars to communicate with sim.c

#define DEFAULT_PINGINTERVAL 5000.0

extern char *ConfigFileName, *SchemaFileName, *WkldFileName, *SimTimeChars;
extern float SimTime;
extern bool CSIMReports; 
extern float PingInterval;
extern bool ResetBufferFlag;
extern char BufMgrType;
extern char DiskType;
extern float GlobalStatWaitSecs;
extern bool PlotMode;

void SetFlags(int argc, char** argv) {
  int i, j, valCnt, val1, val2, val3;
  char* valStr;
  bool skipArg;

  for (i = 1; i < argc; i++) {
    valStr = &(argv[i][2]);                // skip the flag

    // If there is one or more blanks between the flag and the value, then
    // the argument value will be in the next argv[] element.

    if (*valStr == '\0') {
      if (i == argc-1) // last argument?
	valStr = NULL;
      else
	valStr = &(argv[i+1][0]);
      skipArg = TRUE;
    }
    else
      skipArg = FALSE;
    
    // debug trace flag (-d)
    if (strncmp(argv[i], "-d", 2) == 0 && valStr != NULL) {  // trace flag?
      if (strchr(valStr, 'B') != NULL) Flags[TRACE_BUF]     = TRUE;
      if (strchr(valStr, 'D') != NULL) Flags[TRACE_DISK]    = TRUE;
      if (strchr(valStr, 'I') != NULL) Flags[TRACE_IO]      = TRUE;
      if (strchr(valStr, 'T') != NULL) Flags[TRACE_TERM]    = TRUE;
      if (strchr(valStr, 'S') != NULL) Flags[TRACE_SCHED]   = TRUE;
      if (strchr(valStr, 'O') != NULL) Flags[TRACE_OP]      = TRUE;
      if (strchr(valStr, 'M') != NULL) Flags[TRACE_MSG]     = TRUE;
      if (strchr(valStr, 'L') != NULL) Flags[TRACE_LOCK]    = TRUE;
      if (strchr(valStr, 'Q') != NULL) Flags[TRACE_QUERY]   = TRUE;
      if (strchr(valStr, 'P') != NULL) Flags[TRACE_PARSER]  = TRUE;
      if (strchr(valStr, 'C') != NULL) Flags[TRACE_CONTROL] = TRUE;
      if (strchr(valStr, 'V') != NULL) Flags[TRACE_VBUFMGR] = TRUE;
      if (strchr(valStr, 'v') != NULL) Flags[TRACE_VBFUNCS] = TRUE;
      if (strchr(valStr, 'G') != NULL) Flags[TRACE_GOALS]   = TRUE;
      if (strchr(valStr, 'K') != NULL) Flags[TRACE_LEAK]    = TRUE;
      if (strchr(valStr, 'A') != NULL) 
	for (j = 0; j < MAXFLAGS; j++) Flags[j] = TRUE;
    }
	       
    // configuration file name flag (-c)
    else if (strncmp(argv[i], "-c", 2) == 0 && valStr != NULL)
      ConfigFileName = valStr;                // point to (suspected) file name

    // db schema file name flag (-s)
    else if (strncmp(argv[i], "-s", 2) == 0 && valStr != NULL)
      SchemaFileName = valStr;                // point to (suspected) file name

    // workload definition file name flag (-w)
    else if (strncmp(argv[i], "-w", 2) == 0 && valStr != NULL)
      WkldFileName = valStr;                  // point to (suspected) file name

    // reset buffer flag (-R)
    else if (strncmp(argv[i], "-R", 2) == 0) {
      ResetBufferFlag = TRUE;             
      skipArg = FALSE;
    }

    // msg level flag (-l)
    else if (strncmp(argv[i], "-l", 2) == 0 && valStr != NULL)
      sscanf(valStr, "%d", &MsgLevel);        // convert char to int
    
    // Plot mode flag (-P)
    else if (strncmp(argv[i], "-P", 2) == 0) {
      PlotMode = TRUE;
      skipArg = FALSE;
    }

    // buffer manager type flag (-B)
    if (strncmp(argv[i], "-B", 2) == 0 && valStr != NULL) { 
      BufMgrType = *valStr;
    }

    // Global statwaitsecs -- how long to hold off collecting global stats
    else if (strncmp(argv[i], "-x", 2) == 0) {
      sscanf(valStr, "%f", &GlobalStatWaitSecs);
      GlobalStatWaitSecs *= 1000.0 ; //convert seconds to msecs
    }

    // disk type flag (-D)
    if (strncmp(argv[i], "-D", 2) == 0 && valStr != NULL) { 
      DiskType = *valStr;
    }

    // CSIM reports flag (-C)
    else if (strncmp(argv[i], "-C", 2) == 0) {
      CSIMReports = TRUE;
      skipArg = FALSE;
    }
    
    // ping flag (-p)
    else if (strncmp(argv[i], "-p", 2) == 0) {

      // If there is a value specifed, use it, otherwise use default value

      if (valStr != NULL && isdigit(*valStr)) { // next token is a number
        sscanf(valStr, "%f", &PingInterval);
      } else {
	PingInterval = DEFAULT_PINGINTERVAL;
	skipArg = FALSE;
      }
    }
    
    // simulation time flag (-t)
    else if (strncmp(argv[i], "-t", 2) == 0 && valStr != NULL) {
      valCnt = sscanf(valStr, "%d:%d:%d", &val1, &val2, &val3);
      if (valCnt > 0 && valCnt < 4) {
	SimTimeChars = new char[strlen(valStr)+1];
	strcpy(SimTimeChars, valStr);
      }
      if (valCnt == 1)                        // time specified in seconds only
	SimTime = val1 * 1000;                // convert to milliseconds
      else if (valCnt == 2)                   // time specified as mm:ss
	SimTime = val1 * 60000 + val2 * 1000; // convert to milliseconds
      else if (valCnt == 3)                   // time specified as hh:mm:ss
	SimTime = val1 * 3600000 + val2 * 60000 + val3 * 1000; // conv to msecs
    }    

   if (skipArg) i++;

  }  // end for all command line arguments
}

