#include <cpp.h>     
typedef unsigned int size_t;
extern "C" {
    int abs(int);
}

//  Name:        dbsim.h
//
//  Title:       UW database simulator common declarations     
//
//  Change Activity:
//

#define Max(A,B) ( ( (A) > (B) ) ? (A) : (B) )
#define Min(A,B) ( ( (A) < (B) ) ? (A) : (B) )
#undef NULL
#define NULL 0

// Configuration constants.  Additional disk constants are located in disk.h 

#define DEFAULT_NUMTERMINALS     1
#define DEFAULT_NUMNODES        32
#define DEFAULT_BUFSIZE       1024  
#define DEFAULT_DISKSPERNODE 	 1
#define DEFAULT_CPUSPERNODE 	 1
#define DEFAULT_MIPS 	        20
#define DEFAULT_TUPSIZE        200
#define DEFAULT_KBYTESPERNODE 8192
#define DEFAULT_PAGESIZE      8192

#define DEFAULT_MAXPACKETBYTES        8192
#define DEFAULT_GLOBKBYTES               0
#define DEFAULT_LOCKBYTES                0
#define DEFAULT_MINSENDPROTOCOL     1000.0
#define DEFAULT_SENDPROTOCOLPERBYTE    0.0 
#define DEFAULT_MINRECVPROTOCOL     1000.0
#define DEFAULT_RECVPROTOCOLPERBYTE    0.0 
#define DEFAULT_MINDELAY             0.835
#define DEFAULT_DELAYPERBYTE      0.000582
#define DEFAULT_MINSHORTINSTR        773.0
#define DEFAULT_SHORTINSTRPERBYTE    1.181

#define MAX_OPSPERNODE 	       300
#define MAX_TERMINALS         1000
#define MAX_NODES              250
#define MAX_DISKSPERNODE        10
#define MAX_FILES             1500   /* includes relation and index files */
#define MAX_CLASSES            100

#define EXTENT_SIZE             64

// Flag constants

#define TRACE_BUF       0
#define TRACE_DISK      1 
#define TRACE_IO        2
#define TRACE_TERM      3
#define TRACE_SCHED     4
#define TRACE_OP        5
#define TRACE_MSG       6
#define TRACE_LOCK      7  
#define TRACE_QUERY     8
#define TRACE_PARSER    9
#define TRACE_CONTROL  10
#define TRACE_VBUFMGR  11
#define TRACE_VBFUNCS  12
#define TRACE_GOALS    13
#define TRACE_LEAK     14
#define MAXFLAGS       15    /* don't forget to update this guy too! */

// Misc constants
 
#define TERMINAL        0
#define NODE            1
#define SIMNAME         "ZetaSim"   /* for messages */


#define TRUE 1
#define FALSE 0
#define NOTOK 0
#define	OK 1
/* detect and identify error; return if error */
#define CKERR_RET(p,c)  if ((int)(c)!=OK) {\
	printf("%s %d\n", p, c); \
	return(c);\
}

// Type/class definitions

typedef int bool;

extern "C" {
  void exit(int);
}

// Global variables

extern event SimDone;
extern int SimRunning;
extern bool Flags[];
extern bool Quiet;
extern int MsgLevel;
extern int NumClasses;
//extern int NumTerminals;
extern int ActiveTerms;
extern int NumNodes;
extern int NumDiskNodes;
extern int DisksPerNode;
extern int CPUsPerNode;
extern int BufSize;
extern int PageSize;
extern int KBytesPerNode;
extern float Mips;
extern int MaxPacketBytes;
extern int MaxRequests;
extern int GlobKBytes;
extern int LocKBytes;
extern float MinSendProtocol;
extern float MinRecvProtocol;
extern float SendProtocolPerByte;
extern float RecvProtocolPerByte;
extern float MinDelay;
extern float DelayPerByte;
extern float MinShortInstr;
extern float ShortInstrPerByte;
