
#include <dbsim.h>
#include <disk.h>
#include <strings.h>
#include <math.h>

// ---------------------------------------------------------------------------
// class implementation for DiskQueue (queue of disk requests)
// ---------------------------------------------------------------------------

// insert disk request into "in" queue in ascending cylinder order

void DiskQueue::enqueueIn(DiskReq *newReq)
{
    DiskReq *ptr1, *ptr2;

    if (qhead.nextReq == NULL) {
	// queue is empty so just stick the new request at the front
	qhead.nextReq = newReq;
	newReq->nextReq = NULL;
    } else {
	ptr1 = &qhead;
	ptr2 = qhead.nextReq;
	// search for the proper spot to insert the new request
	while ((ptr2 != NULL) && (newReq->cylinder < ptr2->cylinder)) {
	    ptr1 = ptr2;
	    ptr2 = ptr2->nextReq;
	}
	// the new request should go between ptr1 and ptr2
	newReq->nextReq = ptr1->nextReq;
	ptr1->nextReq = newReq;
    }

//    printf("%f(%d) InQ  insert %X cyl %d; after insert, inQ = ",
//            clock, identity(), newReq, newReq->cylinder);
//    print();

}

// insert disk request into "out" queue in descending cylinder order

void DiskQueue::enqueueOut(DiskReq *newReq)
{
    DiskReq *ptr1, *ptr2;

    if (qhead.nextReq == NULL) {
	// queue is empty so just stick the new request at the front
	qhead.nextReq = newReq;
	newReq->nextReq = NULL;
    } else {
	ptr1 = &qhead;
	ptr2 = qhead.nextReq;
	// search for the proper spot to insert the new request
	while ((ptr2 != NULL) && (newReq->cylinder > ptr2->cylinder))
	{
	    ptr1 = ptr2;
	    ptr2 = ptr2->nextReq;
	}
	// the new request should go between ptr1 and ptr2
	newReq->nextReq = ptr1->nextReq;
	ptr1->nextReq = newReq;
    }

//    printf("%f(%d) OutQ insert %X cyl %d; after insert, inQ = ",
//            clock, identity(), newReq, newReq->cylinder);
//    print();

}

void DiskQueue::print()
{
    DiskReq *ptr;

    ptr = qhead.nextReq;
    printf("<  ");
    while (ptr != NULL) {
	printf("%X(%d)  ",ptr,ptr->cylinder);
	ptr = ptr->nextReq;
    }
    printf(">\n");
}

