#include "dbsim.h"
#include "object.h"
#include "node.h"
#include "msg.h"
#include "mailbox.h"
#include "split.h"
#include "cost.h"
#include "op.h"
#include "query.h"

extern "C" {
  void exit(int);
}

int SplitTable::numSplitTables = 0;
SplitTable* SplitTable::freeSplitTable = NULL;

// constructor function
SplitTable::SplitTable(
  int numEntries, 
  int tupSize, 
  bool broadcast) {

splitCnt = numEntries;
fullBroadcast = broadcast;
tupleSize = tupSize;
tupsPerNetPageOut = NETPKTSIZE/(tupleSize + 15);

splitArray = new NetBufRec[numEntries];

for(int i=0; i<splitCnt; splitArray[i++].numTuples=0);
};

SplitTable::~SplitTable() {
  delete [] splitArray;
//  delete [splitCnt] splitArray; kbaix
}


  // puts numTuples tuples in the split table, and takes care of sending 
  // messages out when a page becomes full.
  // Case 1: fullBroadcast == TRUE, which means that messages are broadcast to 
  //         all other nodes.
  // Case 2: fullBroadcast == FALSE, which means that the message should go to 
  //         only 1 node.
  void SplitTable::putTuples(Node* n, Mailbox* source, bool eofFlag, 
                             int numTuples)
  {
  int i,numPages,numTuplesInEntry,splitIdx;

    // All nodes receive all messages.
    if (fullBroadcast) {
      // calculate number of pages to send out.
      // also need to hold on to the old number of tuples in the buffer
      // because sendNetBuffer resets it to 0.
      numTuplesInEntry = splitArray[0].numTuples;
      numPages = (numTuplesInEntry + numTuples) / tupsPerNetPageOut;

      for (i=0; i<numPages; i++) {
        // send full pages to all nodes. 
        for (splitIdx=0; splitIdx<splitCnt; splitIdx++) {
          splitArray[splitIdx].numTuples = tupsPerNetPageOut;
            if (Flags[TRACE_OP]) {
              printf("%f(%d) op on node %d sends page (%d tuples) to %d\n",
		     clock, identity(), n->id, splitArray[splitIdx].numTuples,
  		     splitArray[splitIdx].destNode->owningNode->id);
            }
          sendNetBuffer(splitIdx,eofFlag,n,source);
        }
      }
        
      // keep any left over tuples, and send later when page is full.
      for (splitIdx=0; splitIdx<splitCnt; splitIdx++) {
        splitArray[splitIdx].numTuples = (numTuplesInEntry + numTuples) 
                                  % tupsPerNetPageOut;
      }
    }

    // Only 1 node receives message.
    else {
        splitIdx = random(0, splitCnt-1);

      // calculate number of pages to send out.
      // also need to hold on to the old number of tuples in the buffer
      // because sendNetBuffer resets it to 0.
      numTuplesInEntry = splitArray[splitIdx].numTuples;
      numPages = (splitArray[splitIdx].numTuples + numTuples) 
                   / tupsPerNetPageOut;

//        if (Flags[TRACE_OP]) {
//        printf("split table index entry (%d) now has (%d tuples) \n", splitIdx, 
//                splitArray[splitIdx].numTuples + numTuples);
//        }

      for (i=0; i<numPages; i++) {
        //split buffer is full, so send it to destination
        splitArray[splitIdx].numTuples = tupsPerNetPageOut;
          if (Flags[TRACE_OP]) {
          printf("%f(%d) op node on %d sends page (%d tuples) to node %d\n",
		 clock, identity(), n->id, splitArray[splitIdx].numTuples,
		 splitArray[splitIdx].destNode->owningNode->id,clock);
          }
        sendNetBuffer(splitIdx,eofFlag,n,source);
      }

      // keep any left over tuples, and send later when page is full.
      splitArray[splitIdx].numTuples = (numTuplesInEntry + numTuples) 
                                          % tupsPerNetPageOut;
//        if (Flags[TRACE_OP]) {
//        printf("index entry (%d) now has (%d tuples) leftover \n", splitIdx, 
//                splitArray[splitIdx].numTuples);
//        }
    }
  }                 


  void SplitTable::sendNetBuffer(int bufIdx, bool eofFlag, Node* n, 
			       Mailbox* source)
  {
      register DataMsg    *dMsg;
      if (n); // make compiler happy

      dMsg = new DataMsg;  // allocate a free buffer
      dMsg->numTuples = splitArray[bufIdx].numTuples;
      dMsg->tupleSize = tupleSize;
      dMsg->eofFlag = eofFlag;

      if ((bufIdx < 0) || (bufIdx >= splitCnt))
        printf("SendNetBuffer: Illegal bufIdx=%d, splitCnt=%d\n",
                bufIdx, splitCnt);
    
      splitArray[bufIdx].numTuples = 0;
      // now send off the message
      dMsg->send(splitArray[bufIdx].destNode, source);

//      if (Flags[TRACE_OP]) {
//        printf(" sending page to %d  \n", bufIdx); 
//      }
  }

  void SplitTable::flushNetBufs(Node* n, Mailbox* source)
  {
      register int  i;
      for (i=0;i<splitCnt;i++) sendNetBuffer(i, TRUE, n, source);
  }
