%{
#define isparser "yup"
#include <stdio.h>
#include <math.h>
#include <limits.h>
#include "string.h"
#include "dbsim.h"
#include "object.h"
#include "node.h"
#include "query.h"
#include "language.h"
#include "disk.h"
#include "db.h"
#include "cost.h"

///////////////////////////////////////////////////////////////////////////////
//
//    Name:       parser.y (yyparse, yyerror, yywhere, yywrap)
//  
//    Function:   A lex/yacc generated parser which translates hardware and 
//                software configuration, database definition, and workload
//                definition statements into "template objects" (see 
//                language.h).  These template objects are then used to cReate 
//                specific instances of simulator objects:  nodes, disks, 
//                terminals, and queries.
//
//    Change Activity:
//
//     09/91 - Original version.  KPB
//  11/01/91 - Added index support.  KPB
//
///////////////////////////////////////////////////////////////////////////////

extern SymTable   RealSyms;      // real num symbol table
extern SymTable   FileSyms;      // file symbol table
extern SymTable   PlanSyms;      // plan symbol table
extern SymTable   JobClassSyms;  // job class symbol table
extern Node       *Nodes[MAX_NODES];
extern Index      *Indices[MAX_FILES];
extern File       *Relations[MAX_FILES];
extern JobClass   *JobClasses[MAX_CLASSES];
extern bool       OpenClasses;   // one or more open classes exist
extern int*       AllDiskIDs;    // array of one integer per DisksPerNode
extern bool       PrimaryClassNeeded, PrimaryClassFound;
extern int        Inst_StartIO;

class IntegerSet;

ProbDist   *currProbDist; // working ptr to a user specified prob distr array
ProbDist   *currProbDist2;// working ptr to a user specified prob distr array
RandomReal *currRealNum;  // working ptr to a random real number 
RandomFile *currFile;     // working ptr to a random file 
RandomPlan *currPlan;     // working ptr to a random plan 
JobClass   *currJClass;   // working ptr to a job class 
IntegerSet *currIntSet;   // working ptr to a set of integers
MemReq     *currMemReq;   // working ptr to a memory requirement
int        i;
bool       stmtErr = FALSE; // some syntax/semantic error on current stmt

static char errmsg[80];
extern char yytext[];     // lex managed, holds current input file line
extern int yyleng;
extern int yylineno;
FILE*      yyerfp = stdout;
extern char *SourceFileName;

void yyerror(char* s);
void       yywhere();
int        yylook();
int        yyback(int*, int);
int        yywrap();
void       CreateFile(char* sym, RandomFile* f, int fileID);
void       ResetFile();
void       CreateJobClass(char* sym, JobClass* jc);
void       CreatePlan(char* sym, RandomPlan* p);

struct planInfo {         // place to accumulate plan attributes during parse
  bool stats;
  int  maxmpl;
} currPlanInfo = {FALSE, -1};

struct fileinfo {         // place to accumulate file attributes during parse
  fileType type;          // index_file or data_file
  int      fileNo;
  IntegerSet* nodeSet;
  IntegerSet* diskSet;
  int      tupleSize;
  int      numTuples;
  int      keySize;
  int      indexType;
  float    probHit;
  RandomFile* indexedFile; 
} currFileInfo =  {data_file, -1, NULL, NULL, -1, -1, -1, -1, 0.0, NULL};

typedef enum intset_type {random_intset, constant_intset};

class IntegerSet {          // random or fixed set of integers
  public:			    
    intset_type setType;    // random or constant			      
    int      max;           // max cardinality and element value allowed
    int      card;          // current cardinality (number of elements)
    int*     set;           // actual set (array) of integers
    RandomSet* randSet;     // points to RandomSet if specified that way

    IntegerSet() {

      // Prepare for default of constant_intset and max unknown

      setType = constant_intset;
      max = 0;  
      card = 0;  
      set = NULL;
      randSet = NULL;
    };

    IntegerSet(int mx) {

      // Prepare for default of constant_intset

      setType = constant_intset;
      max = mx;  
      card = 0;  
      set = new int[max];
      randSet = NULL;
    };

    ~IntegerSet() {delete set;};

    int add(int e) {
      if ((int) e < max && card < max) {
	set[card++] = (int) e;
	return(0);          // indicate successful
      }
      else {
	sprintf(errmsg, "max cardinality is %d and max element value is %d for this set", max, max-1);
	yyerror(errmsg);
	return(-1);         // indicate some problem
      }
    };
    
    // add the sites on which file declustered to the integer set
    int add(File* file) {
        int i;
	for (i = 0; i < file->numNodes; i++) {
	  this->add(file->storageNode(i)->id);
	}
	return(0);          // indicate successful
    }
};

%}

/* Union of all the (compatible) types that can live on the semantic stack */

%union {               
        int intval;
	float floatval;
	bool boolval;
	char *string;
	scanTypes stype;
        ProbDist* probdist;
        RandomReal* randreal;
        RandomFile* randfile;
	RandomPlan* randplan;
        JobClass* jclass;
	IntegerSet* iset;
	MemReq* mreq;
        }

/* Mappings of object types to non-terminals which detect those object types */

%type<probdist> ProbDist
%type<randreal> RealNum
%type<randfile> File
%type<randplan> Plan PlanList Query QueryOp
%type<jclass> JobClass
%type<stype> ScanType
%type<iset> Sites
%type<mreq> MemReq
%type<intval> MemUnit

/* Tokens recognized by the lexical analyzer (tokenizer.l) */

%token <string> FILENAME
%token <string> NAME
%token <floatval> NUMBER
%token SEMI COMMA EQUALS SLASH DOTDOT SPLAT
%token LPAREN RPAREN LBRACK RBRACK LBRACE RBRACE LESSTHAN GREATERTHAN
%token REAL PLAN FILEKW INDEXKW JOBCLASS INTERARRIVALSECS
%token PROJ SORT AGGR XPROD UPDATE WHERE INDEXSCAN FILESCAN STOREOP
%token HASHJOIN INDEXREPLICATEJOIN INDEXREPLICATEJOINP 
%token UNION INTERSECTION DIFFERENCE TERMSEND UNIQUE SORTUNIQUE
%token EXPNTL NORMAL UNIFORM RANDOM SITES
%token ID TUPLESIZE NUMTUPLES DISKS NODES KEYSIZE TYPE CLUSTEREDKW PROBHIT 
%token UNCLUSTEREDKW THINKSECS POPULATION PLANS CPUS CPUMIPS KBYTES PAGESIZE
%token PAGESPERCYL SEEKFACTOR MINLATENCY ROTATION SETTLE XFERRATE PRIMARY
%token CACHEPAGES BPOOL MIN MAX BYTES KBYTES MBYTES MAXMPL STATS CACHECONTEXTS
%token STARTIO_INST NETWORK MAXPACKETBYTES MAXREQUESTS GLOBKBYTES
%token LOCKBYTES MINSENDPROTOCOL SENDPROTOCOLPERBYTE MINRECVPROTOCOL 
%token RECVPROTOCOLPERBYTE MINDELAY DELAYPERBYTE MINSHORTINSTR 
%token SHORTINSTRPERBYTE WAITSECS STATWAITSECS
%token MAXCOMPLETIONS 

/* Goal symbol of the language */

%start SimDef

/* Productions and their associated semantics (actions) within the braces */

%%
 SimDef:         Stmt
             |   SimDef Stmt { yyerrok; stmtErr = FALSE; }
             ;

 Stmt:           RealNumStmt
             |   FileStmt
             |   JobClassStmt
             |   PlanStmt
             |   NodeStmt
             |   NetworkStmt
             |   DiskStmt

             |   error Semi { yyerror("unknown statement type"); }
             ;

 ProbDist:       LBRACK { currProbDist = new ProbDist(rand_dist); } 
                 ProbList RBRACK {
                   if (!currProbDist->spread()) // if problem spreading prob
		     delete currProbDist;    // dispose of defective list
		   else 
		     $$ = currProbDist;      // push the list on semantic stack
		 }

             |   LESSTHAN { currProbDist = new ProbDist(determ_dist); } 
                 ProbList GREATERTHAN {
	           $$ = currProbDist;        // push the list on semantic stack
		 }

             |   LBRACK { currProbDist = new ProbDist(rand_dist); } 
                 SITES LPAREN NAME RPAREN RBRACK {
		      currFile = (RandomFile*) FileSyms.lookup($5); 
		      if (currFile == NULL) { // no such symbol?
			 sprintf(errmsg, "undefined file:  %s", $5);
		         yyerror(errmsg);
		      } else {
			 currProbDist->add(currFile->file);
		      }
		      if (!currProbDist->spread()) // if problem spreading prob
			  delete currProbDist;    // dispose of defective list
		      else 
			  $$ = currProbDist; // push the list on semantic stack
		 }
             ;

 ProbList:       ProbListEl
             |   ProbList COMMA ProbListEl
             ;

 ProbListEl:     NAME SLASH NUMBER 
                   { currProbDist->add($1, $3); }

             |   NUMBER SLASH NUMBER 
                   { currProbDist->add($1, $3); }

             |   NAME SPLAT NUMBER 
                   { currProbDist->add($1, (int) $3); }

             |   NUMBER SPLAT NUMBER 
                   { currProbDist->add($1, (int) $3); }

             |   NAME 
                   { currProbDist->add($1); }

             |   NUMBER
                   { currProbDist->add($1); }
             ;

 RealNumStmt:    REAL RealNumList Semi
             |   REAL error {yyerror("bad real statement syntax");}
             ;

 RealNumList:    NAME EQUALS RealNum {
                   if (!stmtErr)
		     if (!RealSyms.add($1, $3))
		       delete $3;             // some error adding symbol
		   stmtErr = FALSE;
		 }

             |   RealNumList COMMA NAME EQUALS RealNum {
                   if (!stmtErr)
		     if (!RealSyms.add($3, $5))
		       delete $5;             // some error adding symbol
		   stmtErr = FALSE;
		 }
             ;

 RealNum:        NUMBER {
                   currRealNum = new RandomReal;
		   currRealNum->dist = const_real;
		   currRealNum->val = $1;
		   $$ = currRealNum;     // push RandomReal on semantic stack  
		 }

             |   NAME {
                   currRealNum = (RandomReal*) RealSyms.lookup($1);
		   if (currRealNum == NULL) { // no such symbol?
		     sprintf(errmsg, "undefined real number:  %s", $1);
		     yyerror(errmsg);
		   }
		   else
		     $$ = currRealNum;     // push RandomReal on semantic stk
		 }

             |   UNIFORM LPAREN NUMBER COMMA NUMBER RPAREN {
                   currRealNum = new RandomReal;
		   currRealNum->dist = uniform_real;
		   currRealNum->uniformLow = $3;
		   currRealNum->uniformHigh = $5;
		   $$ = currRealNum;     // push RandomReal on semantic stack  
		 }

             |   RANDOM LPAREN NUMBER COMMA NUMBER RPAREN {
                   currRealNum = new RandomReal;
		   currRealNum->dist = rand_real;
		   currRealNum->uniformLow = $3;
		   currRealNum->uniformHigh = $5;
		   $$ = currRealNum;     // push RandomReal on semantic stack  
		 }

             |   EXPNTL LPAREN NUMBER RPAREN {
                   currRealNum = new RandomReal;
		   currRealNum->dist = expntl_real;
		   currRealNum->mean = $3;
		   $$ = currRealNum;     // push RandomReal on semantic stack  
		 }

             |   NORMAL LPAREN NUMBER COMMA NUMBER RPAREN {
                   currRealNum = new RandomReal;
		   currRealNum->dist = nrm_real;
		   currRealNum->mean = $3;
		   currRealNum->variance = $5;
		   $$ = currRealNum;     // push RandomReal on semantic stack  
		 }

             |   ProbDist {
	           currProbDist = $1;    // grab distr from top of stack
	           if (currProbDist->allReals()) { // are all elements reals? 
	             currRealNum = new RandomReal;
		     currRealNum->dist = userDist_real;
		     currRealNum->userDist = currProbDist;
		     $$ = currRealNum;   // push RandomReal on semantic stack
		     }
		   else {                // at least one object not a real
		     yyerror("one or more undefined real nums in distribution");
		     delete currProbDist; // trash it
		   }
		 }
             ;

 FileStmt:       FILEKW {currFileInfo.type = data_file;} FileList Semi
             |   INDEXKW {currFileInfo.type = index_file;} FileList Semi
             |   FILEKW error {yyerror("bad file statement syntax");}
             |   INDEXKW error {yyerror("bad index statement syntax");}
             ;

 FileList:       FileListEl
             |   FileList COMMA FileListEl
             ;

 FileListEl:     NAME EQUALS File { 
                   CreateFile($1, $3, 0); 
		   ResetFile();           // clear accumulated attributes
		 }

             |   NUMBER EQUALS File { 
	           currFile = $3;
		   if (currFile->dist != const_file)
		     yyerror("only constant files may be unnamed");
		   else {
		     if (notafile((int) $1))
		       CreateFile(NULL, $3, (int) $1); 
		     else
		       yyerror("cannot reuse an existing data/index file ID");
		   }
		   ResetFile();           // clear accumulated attributes
		 }

             |   NUMBER DOTDOT NUMBER EQUALS File {
		   if ($5->dist != const_file)
		     yyerror("only constant files may be unnamed");
		   else {
		     if ($3 > $1) 
		       for (i = (int) $1; i <= (int) $3; i++) {
			 if (stmtErr) break; // dont cascade errors
			 if (notafile(i)) {
			   currFile = new RandomFile; // replicate file
			   currFile->dist = const_file;
			   CreateFile(NULL, currFile, i);
			 }
			 else
			   yyerror("cannot reuse an existing data/index file ID");
		       }
		     else {
		       sprintf(errmsg, "%d..%d is an invalid range", (int) $1, 
			       (int) $3);
		       yyerror(errmsg);
		     }
		   }
		   ResetFile();           // clear accumulated attributes
		   ($5)->fileNo = -1;     // mess up fileNo so destructor only
		                          // cleans up rand file and nothing
					  // else by mistake
		   delete $5;             // only used as a model for clones
		 }
             ;

 File:           NUMBER {
		   if (notafile((int) $1)) {
		     sprintf(errmsg, "undefined file:  %d", (int) $1);
		     yyerror(errmsg);
		   }
		   else 
		     $$ = ConstFiles[(int) $1]; // push file on semantic stack
		 }

             |   NAME {
                   currFile = (RandomFile*) FileSyms.lookup($1); // find symbol
		   if (currFile == NULL){ // no such symbol?
		     sprintf(errmsg, "undefined file:  %s", $1);
		     yyerror(errmsg);
		   }
		   else
		     $$ = currFile;       // push file on semantic stack
		 }

             |   ProbDist {
	           currProbDist = $1;    // grab distr from top of stack
	           if (currProbDist->allFiles(currFileInfo.type)) {
	             currFile = new RandomFile;
		     currFile->dist = userDist_file;
		     currFile->userDist = currProbDist;
		     $$ = currFile;       // push file on semantic stack
		   }
		   else {                 // at least one bad thing in dist
		     delete currProbDist; // trash it
		   }
		 }

             |   UNIFORM LPAREN NUMBER COMMA NUMBER RPAREN {

                   // Check existence of all files in distribution

		   for (i = (int) $3; i <= (int) $5; i++) {
		     if (notafile(i)) {
		       sprintf(errmsg, "range includes an undefined file:  %d",
			       i);
		       yyerror(errmsg);
		       break;
		     }
		     if (ConstFiles[i]->type != currFileInfo.type) {
		       yyerror("range includes both index and data files");
		       break;
		     }
		   }

		   if (i > (int) $5) {        // all files checked out
		     currFile = new RandomFile;
		     currFile->dist = random_file;
		     currFile->randomLow = (int) $3;
		     currFile->randomHigh = (int) $5;
		     $$ = currFile;       // push file on semantic stack
		   }
		 }

             |   LESSTHAN FileAttrList GREATERTHAN {
                   currFile = new RandomFile;
		   currFile->dist = const_file;
		   $$ = currFile;       // push file on semantic stack
		 }
             ;

 FileAttrList:   FileAttr
             |   FileAttrList COMMA FileAttr
  
             |   FileAttrList error {yyerror("bad file/index attribute"); }
             ;

 FileAttr:       ID EQUALS NUMBER {
		   if (notafile((int) $3))
		     currFileInfo.fileNo = (int) $3;
		   else
		     yyerror("cannot reuse an existing data/index file ID");
		 }

              |  TUPLESIZE EQUALS NUMBER {
		   if ($3 <= 0.0)
		     yyerror("tuplesize must be > 0");
		   else
		     currFileInfo.tupleSize = (int) $3;
		 }

              |  NUMTUPLES EQUALS NUMBER {
		   if ($3 <= 0.0)
		     yyerror("number of tuples must be > 0");
		   else
		     currFileInfo.numTuples = (int) $3;
		 }

             |   NODES EQUALS { currIntSet = new IntegerSet(NumDiskNodes); }
                 IntSet { currFileInfo.nodeSet = currIntSet; }

             |   DISKS EQUALS {currIntSet = new IntegerSet(DisksPerNode); }
                 IntSet { currFileInfo.diskSet = currIntSet; }
		     
             |   KEYSIZE EQUALS NUMBER {
	           currFileInfo.keySize = (int) $3;
	         }

             |   TYPE EQUALS CLUSTEREDKW {
	           currFileInfo.indexType = CLUSTERED;
	         }

             |   TYPE EQUALS UNCLUSTEREDKW {
	           currFileInfo.indexType = UNCLUSTERED;
	         }

             |   FILEKW EQUALS NAME {
                   currFile = (RandomFile*) FileSyms.lookup($3); // find symbol
		   
		   if (currFile == NULL){ // no such symbol?
		     sprintf(errmsg, "undefined file:  %s", $3);
		     yyerror(errmsg);
		   }
		   else {
		     currFileInfo.indexedFile = currFile;
		     
		     if (currFile->dist != const_file)
		       yyerror("idices may only be created on constant files");

		     if (currFile->type != data_file)
		       yyerror("idices may only be created on data files");
		   }
	         }

             |   FILEKW EQUALS NUMBER {
		   if (notafile((int) $3)) {
		     sprintf(errmsg, "undefined file:  %d", (int) $3);
		     yyerror(errmsg);
		   }
		   else {
		     currFileInfo.indexedFile = ConstFiles[(int) $3];
		     
		     if (currFileInfo.indexedFile->dist != const_file)
		       yyerror("idices may only be created on constant files");

		     if (currFileInfo.indexedFile->type != data_file)
		       yyerror("idices may only be created on data files");
		   }
	         }
             |   PROBHIT EQUALS NUMBER {
	           currFileInfo.probHit = $3;
		   if ((currFileInfo.probHit < 0) || 
		       (currFileInfo.probHit > 1.0)) {
		     yyerror("prob hit parameter doesn't make sense\n");
		   }
                 }	        
             ;

 IntSet:         LBRACE IntSetList RBRACE
             ;

 IntSetList:     NUMBER { currIntSet->add((int) $1); } 

             |   IntSetList COMMA NUMBER { currIntSet->add((int) $3); } 
   
             |   NUMBER DOTDOT NUMBER {
	           if ((int) $3 >= (int) $1)
		     for (i = (int) $1; i <= (int) $3; i++)
		       if (currIntSet->add(i) != 0) break;
		 }

             |   IntSetList COMMA NUMBER DOTDOT NUMBER { 
	           if ((int) $5 >= (int) $3)
		     for (i = (int) $3; i <= (int) $5; i++)
		       if (currIntSet->add(i) != 0) break;
		 }

             |   RealNum SPLAT RealNum {
	           currIntSet->setType = random_intset; // change from default
		   delete currIntSet->set; // kill fixed set storage
		   currIntSet->set = NULL;
		   currIntSet->randSet = new RandomSet(currIntSet->max);
	           currIntSet->randSet->elements = $1;
	           currIntSet->randSet->cardinality = $3;
		 }

             |   SITES LPAREN NAME RPAREN {
		      currFile = (RandomFile*) FileSyms.lookup($3); 
		      if (currFile == NULL) { // no such symbol?
			 sprintf(errmsg, "undefined file:  %s", $3);
		         yyerror(errmsg);
		      } else {
			 currIntSet->add(currFile->file);
		      }
		 }

             |   NUMBER error {yyerror("bad set definition"); } 
             |   IntSetList error {yyerror("bad set definition"); }
             ;
          
 TimeSeq:        LBRACE { currProbDist  = new ProbDist(determ_dist);
			  currProbDist2 = new ProbDist(determ_dist); }
		 TimeSeqList RBRACE
             ;

 TimeSeqList:    TimeSeqEl
             |   TimeSeqList COMMA TimeSeqEl
             |   TimeSeqEl error {yyerror("bad time sequence list");}
	     ;

 TimeSeqEl:      LPAREN NUMBER COMMA NUMBER RPAREN {
                   if ($2 <= 0.0)
		     yyerror("population must be > 0");
		   else if ($4 <= 0.0)
		     yyerror("time must be > 0");
		   else {
		       currProbDist->add($2);
		       currProbDist2->add($4);
		   }
	         }
	     ;

 JobClassStmt:   JOBCLASS JobClassList Semi
             |   JOBCLASS error {yyerror("bad jobclass statement syntax");}
             ;

 JobClassList:   NAME EQUALS JobClass 
                   { CreateJobClass($1, $3); }

             |   JobClassList COMMA NAME EQUALS JobClass 
                   { CreateJobClass($3, $5); }

             |   NAME error {yyerror("bad job class statement"); }
             |   JobClassList error {yyerror("bad job class statement"); }
             ;

 JobClass:       LESSTHAN {currJClass = new JobClass;} 
                 JClassDefList GREATERTHAN {
		   $$ = currJClass;      // push job class on semantic stack
		 }
             ;

 JClassDefList:  JClassDefStmt
             |   JClassDefList COMMA JClassDefStmt
  
             |   JClassDefList error {yyerror("bad job class definition list");}
             ;

 JClassDefStmt:  POPULATION EQUALS NUMBER {
		   if ($3 < 0.0)
		     yyerror("population must be >= 0");
		   else if ($3 > MAX_TERMINALS) {
		     sprintf(errmsg, "population greater than MAX_TERMINALS");
		     yyerror(errmsg);
		   } else
		     currJClass->population = (int) $3;
		 }

             |   PRIMARY {

	           /* note we cannot set stopCount to the population at this
		      point (which is what it is supposed to be), since 
		      the POPULATION= keyword may not have been recognized
		      yet.  Instead, we just change the value to something
		      non-negative, and the JobClass::start method will 
		      set its value to equal the job class population before
		      kicking off the terminals of this class. */

                   currJClass->stopCount = 0;
		 }

             |   MAXCOMPLETIONS EQUALS NUMBER {
		   if ($3 <= 0)
		     yyerror("completions must be > 0");
		   else
		     currJClass->maxCompletions = (int) $3;
	         }

             |   MAXMPL EQUALS NUMBER {
	           currJClass->maxMPL = (int) $3;
		 }


             |   WAITSECS EQUALS RealNum {
	           currJClass->waitSecs = $3;
		 }

             |   STATWAITSECS EQUALS NUMBER {
	           currJClass->statWaitSecs = $3;
		 }

             |   THINKSECS EQUALS RealNum {
	           currJClass->thinksecs = $3;
		 }

             |   INTERARRIVALSECS EQUALS RealNum {
		   currJClass->interArrivalSecs = $3;
		   OpenClasses = TRUE; // let sim.c know open classes exist
		 }

             |   PLANS EQUALS ProbDist {
		   currProbDist = $3;
		   if (currProbDist->allPlans()) { // are all objects plans?
		     currJClass->plans = currProbDist;       
		   } else {
		     yyerror("one or more undefined plans in distribution");
		     delete currProbDist;         // trash it
		   }
		 }
             ;

 PlanStmt:       PLAN PlanList Semi
             |   PLAN error {yyerror("bad plan statement syntax");}
             ;

 PlanList:       NAME PlanAttrs EQUALS Plan 
                   { CreatePlan($1, $4);
		     ResetPlan();
		   }
             |   PlanList COMMA NAME PlanAttrs EQUALS Plan
                   { CreatePlan($3, $6);
		     ResetPlan();
		   }

             |   NAME error {yyerror("bad plan statement"); }
             |   PlanList error {yyerror("bad plan statement"); }
             ;

 /* If more plan attributes are defined, then this production should be a
    real list, which would return some structure containing all the attributes
    as its value.  Right now, it just returns a boolean, since there is only
    one plan attribute (stats) which has a boolean value. */

 PlanAttrs:  LPAREN PlanAttrList RPAREN
             |
             ;

 PlanAttrList:   PlanAttr
             |   PlanAttrList COMMA PlanAttr
             |   PlanAttrList error {yyerror("bad plan attribute"); }
             ;

 PlanAttr:       STATS {
                     currPlanInfo.stats = TRUE;
		 }

             |   MAXMPL EQUALS NUMBER {
	             currPlanInfo.maxmpl = (int) $3;
		 }
             ;

 Plan:           Query {
                   $$ = $1;  // just pass it on up
		 }

             |   UPDATE File WHERE Query {
	           yyerror("statement not supported yet");
		 }
             ;

 Query:          LPAREN Query RPAREN {
                   $$ = $2;             // just pass it on up
		 }

             |   NAME {
		   if (($$ = (RandomPlan*) PlanSyms.lookup($1)) == NULL) {
		     sprintf(errmsg, "undefined plan:  %s", $1);
		     yyerror(errmsg);
		   }
		 }

             |   ProbDist {
                   currProbDist = $1;
	           if (currProbDist->allPlans()) { // are all objects plans?
		     currPlan = new RandomPlan;
		     currPlan->dist = userDist_plan;
		     currPlan->userDist = currProbDist;
		     $$ = currPlan;
		   }
		   else {                 
		     yyerror("one or more undefined plans in distribution");
		     delete currProbDist;     // trash it
		     }
		 }

             |   QueryOp { 
	           $$ = $1;  // just pass it on up
		 }
             ;

 QueryOp:        PROJ Sites MemReq LPAREN Query COMMA RealNum RPAREN {
                   // proj(Query, retain %) 
	           
		   yyerror("statement not supported yet");
                 }
 
             |   ScanType Sites MemReq LPAREN File RPAREN {
                   //  no selection predicate, no projection 
		   
		   if ($1 == file_scan && $5->type != data_file)
		     yyerror("cannot perform a file scan on an index file");
		   if ($1 == index_scan && $5->type != index_file)
		     yyerror("cannot perform an index scan on a data file");

		   currPlan = new RandomPlan;
		   currPlan->dist = const_plan;
		   currPlan->opCode = SELECT;
		   currPlan->file = $5;
		   currPlan->sel = new RandomReal;
		   currPlan->sel->dist = const_real;
		   currPlan->sel->val = 1.0;
		   if ($1 == index_scan) { //need to default index selectivity?
		     currPlan->isel = new RandomReal;
		     currPlan->isel->dist = const_real;
		     currPlan->isel->val = 1.0;
		   }
		   currPlan->proj = new RandomReal;
		   currPlan->proj->dist = const_real;
		   currPlan->proj->val = 1.0;
		   if ($2 != NULL) {       // if site assmt expr present
		     currPlan->sites = ($2)->set;
		     currPlan->sizeSites = ($2)->card;
		     currPlan->siteSet = ($2)->randSet;
		   }
		   currPlan->memReq = $3;
		   $$ = currPlan;
                 }

             |   ScanType Sites MemReq LPAREN File COMMA RealNum RPAREN {
                   // fscan(File, selectivity %) 
		   
		   if ($1 == file_scan && $5->type != data_file)
		     yyerror("cannot perform a file scan on an index file");
		   if ($1 == index_scan && $5->type != index_file)
		     yyerror("cannot perform an index scan on a data file");
		   if ($1 == index_scan)
		     yyerror("index scan requires 2 selectivities:  operator sel and index sel");

		   currPlan = new RandomPlan;
		   currPlan->dist = const_plan;
		   currPlan->opCode = SELECT;   
		   currPlan->file = $5;
		   currPlan->sel = $7;
		   currPlan->proj = new RandomReal;
		   currPlan->proj->dist = const_real;
		   currPlan->proj->val = 1.0;
		   if ($2 != NULL) {       // if site assmt expr present
		     currPlan->sites = ($2)->set;
		     currPlan->sizeSites = ($2)->card;
		     currPlan->siteSet = ($2)->randSet;
		   }
		   currPlan->memReq = $3;
		   $$ = currPlan;
                 }

             |   ScanType Sites MemReq LPAREN File COMMA RealNum COMMA 
                 RealNum RPAREN{
                   // fscan(File, selectivity %, project retain %) -or-
                   // iscan(File, op selectivity %, index selectivity %)
		   
		   if ($1 == file_scan && $5->type != data_file)
		     yyerror("cannot perform a file scan on an index file");
		   if ($1 == index_scan && $5->type != index_file)
		     yyerror("cannot perform an index scan on a data file");

		   currPlan = new RandomPlan;
		   currPlan->dist = const_plan;
		   currPlan->opCode = SELECT;   
		   currPlan->file = $5;
		   currPlan->sel = $7;
		   if ($1 == file_scan)
		     currPlan->proj = $9;
		   else {
		     currPlan->isel = $9;  // must default projection to 100%
		     currPlan->proj = new RandomReal;
		     currPlan->proj->dist = const_real;
		     currPlan->proj->val = 1.0;
		   }
		   if ($2 != NULL) {       // if site assmt expr present
		     currPlan->sites = ($2)->set;
		     currPlan->sizeSites = ($2)->card;
		     currPlan->siteSet = ($2)->randSet;
		   }
		   currPlan->memReq = $3;
		   $$ = currPlan;
                 }

             |   ScanType Sites MemReq LPAREN File COMMA RealNum COMMA RealNum 
                 COMMA RealNum RPAREN {
                   // iscan(File, op sel %, index sel %, proj %)
		   
		   if ($1 == file_scan)
		     yyerror("illegal file scan syntax");
		   if ($1 == index_scan && $5->type != index_file)
		     yyerror("cannot perform an index scan on a data file");

		   currPlan = new RandomPlan;
		   currPlan->dist = const_plan;
		   currPlan->opCode = SELECT;   
		   currPlan->file = $5;
		   currPlan->sel = $7;
		   currPlan->isel = $9;
		   currPlan->proj = $11;
		   if ($2 != NULL) {       // if site assmt expr present
		     currPlan->sites = ($2)->set;
		     currPlan->sizeSites = ($2)->card;
		     currPlan->siteSet = ($2)->randSet;
		   }
		   currPlan->memReq = $3;
		   $$ = currPlan;
                 }

             |   STOREOP Sites MemReq LPAREN Query RPAREN {
	           // store(query) - write query result in new file on disk 0

		   currPlan = new RandomPlan;
		   currPlan->dist = const_plan;
		   currPlan->opCode = STORE;
		   currPlan->lchild = $5;  // unary ops use left child
		   currPlan->disk = new RandomReal; // default is random disk
		   currPlan->disk->dist = rand_real; 
		   currPlan->disk->uniformLow = 0;
		   currPlan->disk->uniformHigh = DisksPerNode-1;
		   if ($2 != NULL) {       // if site assmt expr present
		     currPlan->sites = ($2)->set;
		     currPlan->sizeSites = ($2)->card;
		     currPlan->siteSet = ($2)->randSet;
		   }
		   currPlan->memReq = $3;
		   $$ = currPlan;
		 }

             |   STOREOP Sites MemReq LPAREN Query COMMA RealNum RPAREN {
	           // store(query, disk #) - write query result in new file

		   currPlan = new RandomPlan;
		   currPlan->dist = const_plan;
		   currPlan->opCode = STORE;
		   currPlan->lchild = $5;  // unary ops use left child
		   currPlan->disk = $7;    
		   if ($2 != NULL) {       // if site assmt expr present
		     currPlan->sites = ($2)->set;
		     currPlan->sizeSites = ($2)->card;
		     currPlan->siteSet = ($2)->randSet;
		   }
		   currPlan->memReq = $3;
		   $$ = currPlan;
		 }

             |   TERMSEND Sites MemReq LPAREN Query RPAREN {
	           // termsend(query) - send query result to terminal

		   currPlan = new RandomPlan;
		   currPlan->dist = const_plan;
		   currPlan->opCode = TERMOP;
		   currPlan->lchild = $5;  // unary ops use left child
		   if ($2 != NULL) {       // if site assmt expr present
		     currPlan->sites = ($2)->set;
		     currPlan->sizeSites = ($2)->card;
		     currPlan->siteSet = ($2)->randSet;
		   }
		   if ($3 != NULL) delete $3;  // memory specs are ignored
		   $$ = currPlan;
		 }

             |   AGGR Sites MemReq LPAREN Query COMMA RealNum RPAREN {
                   // aggr(Query, cost/tuple) 
	           
		   yyerror("statement not supported yet");
	         }

             |   SORT Sites MemReq LPAREN Query RPAREN {
                   // sort(query) 
	           
		   yyerror("statement not supported yet");
	         }

             |   SORTUNIQUE Sites MemReq LPAREN Query RPAREN {
                   // sortunique(query) 
	           
		   yyerror("statement not supported yet");
	         }

             |   UNIQUE Sites MemReq LPAREN Query RPAREN {
                   // unique(query) 
	           
		   yyerror("statement not supported yet");
	         }

             |   HASHJOIN Sites MemReq LPAREN Query COMMA Query COMMA RealNum 
                 RPAREN {
                   // hjoin(query, query, join probability) 
		   
		   currPlan = new RandomPlan;
		   currPlan->dist = const_plan;
		   currPlan->opCode = JOIN;   // more specific please!!!
		   currPlan->lchild = $5;
		   currPlan->rchild = $7;
		   currPlan->sel = $9;
		   currPlan->proj = new RandomReal; // default proj% to 100%
		   currPlan->proj->dist = const_real;
		   currPlan->proj->val = 1.0;
		   if ($2 != NULL) {       // if site assmt expr present
		     currPlan->sites = ($2)->set;
		     currPlan->sizeSites = ($2)->card;
		     currPlan->siteSet = ($2)->randSet;
		   }
		   currPlan->memReq = $3;
		   $$ = currPlan;
	         }

             |   HASHJOIN Sites MemReq LPAREN Query COMMA Query COMMA RealNum 
                 COMMA RealNum RPAREN {
                   // hjoin(query, query, join probability, project retain %) 
		   
		   currPlan = new RandomPlan;
		   currPlan->dist = const_plan;
		   currPlan->opCode = JOIN;   // more specific please!!!
		   currPlan->lchild = $5;
		   currPlan->rchild = $7;
		   currPlan->sel = $9;
		   currPlan->proj = $11;
		   if ($2 != NULL) {       // if site assmt expr present
		     currPlan->sites = ($2)->set;
		     currPlan->sizeSites = ($2)->card;
		     currPlan->siteSet = ($2)->randSet;
		   }
		   currPlan->memReq = $3;
		   $$ = currPlan;
	         }

             |   XPROD Sites MemReq LPAREN Query COMMA Query RPAREN {
                   // xprod(query, query) 
	           
		   yyerror("statement not supported yet");
	         }

             |   UNION Sites MemReq LPAREN Query COMMA Query RPAREN {
                   // union(query, query) 
	           
		   yyerror("statement not supported yet");
	         }

             |   INTERSECTION Sites MemReq LPAREN Query COMMA Query COMMA 
                 RealNum RPAREN {
                   // intersect(query, query, % of union) 
	           
		   yyerror("statement not supported yet");
	         }

             |   DIFFERENCE Sites MemReq LPAREN Query COMMA Query COMMA 
                 RealNum RPAREN {
                   // difference(query, query, % of first file retained) 
	           
		   yyerror("statement not supported yet");
	         }

             |   PROJ error {yyerror("bad project syntax");}
             |   ScanType error {yyerror("bad scan syntax");}
             |   AGGR error {yyerror("bad aggregation syntax");}
             |   SORT error {yyerror("bad sort syntax");}
             |   SORTUNIQUE error {yyerror("bad sortunique syntax");}
             |   UNIQUE error {yyerror("bad unique syntax");}
             |   HASHJOIN error {yyerror("bad hash join syntax");}
             |   INDEXREPLICATEJOIN error 
                   {yyerror("bad index replicate join syntax");}
             |   XPROD error {yyerror("bad xprod syntax");}
             |   UNION error {yyerror("bad union syntax");}
             |   INTERSECTION error {yyerror("bad intersection syntax");}
             |   DIFFERENCE error {yyerror("bad difference syntax");}
             ;

 ScanType:       INDEXSCAN { $$ = index_scan; }
             |   FILESCAN  { $$ = file_scan; }
             ;

 Sites:          LBRACE { currIntSet = new IntegerSet(NumDiskNodes); }
                 IntSetList RBRACE {

		   // Verifies the numbers are reasonable for node IDs
		   // Should really check for duplicate node IDs too !!!!

		   if (currIntSet->setType == constant_intset) {

		     for (i = 0; i < currIntSet->card; i++) 
		       if (notanode(currIntSet->set[i])) break;

		     if (i < currIntSet->card) { // some node didnt check out
		       yyerror("one or more invalid nodes specified");
		       delete currIntSet;
		       $$ = NULL;           // tell op to take defaults
		     } else {
		       $$ = currIntSet;     // give op the set of sites
		     }
		   } else {
		     $$ = currIntSet;       // cant verify random sets
		   }
		 }

             |   {$$ = NULL;} /* user wants default site assignment */
             ;
  
 MemReq:         LBRACK RealNum COMMA RealNum RBRACK {
                   yyerror("obsolete memory syntax");
		 }

             |   LBRACK RealNum MemUnit RBRACK {
                   currMemReq = new MemReq;
		   currMemReq->min = currMemReq->max = $2;
		   currMemReq->minUnits = currMemReq->maxUnits = $3;
		   $$ = currMemReq;
		 }

             |   LBRACK RealNum MemUnit COMMA RealNum MemUnit RBRACK {
                   currMemReq = new MemReq;
		   currMemReq->min = $2;
		   currMemReq->minUnits = $3;
		   currMemReq->max = $5;
		   currMemReq->maxUnits = $6;
		   $$ = currMemReq;
		 }

             |   {$$ = NULL;} /* user wants default memory requirements */
             ;

 MemUnit:        BPOOL {$$ = BUFPOOL_UNITS;}
             |   MIN {$$ = MINREQ_UNITS;}
             |   MAX {$$ = MAXREQ_UNITS;}
             |   BYTES {$$ = BYTE_UNITS;}
             |   KBYTES {$$ = KBYTE_UNITS;}
             |   MBYTES {$$ = MBYTE_UNITS;}

 NodeStmt:       NODES EQUALS LESSTHAN NodeAttrList GREATERTHAN SEMI
             |   NODES error {yyerror("bad nodes statement syntax");}
             ;

 NodeAttrList:   NodeAttr
             |   NodeAttrList COMMA NodeAttr
  
             |   NodeAttrList error {yyerror("bad node definition syntax"); }
             ;

 NodeAttr:       POPULATION EQUALS NUMBER {
                   if ((int) $3 > MAX_NODES-1 || (int) $3 <= 0) {
     		     sprintf(errmsg, "number of disk nodes must be between 0 and %d", MAX_NODES);
		     yyerror(errmsg);
		   }
		   else {
                     NumDiskNodes = (int) $3; 
		     NumNodes = NumDiskNodes+1;
		   }
		 }
		     
             |   DISKS EQUALS NUMBER { 
	           if ((int) $3 > MAX_DISKSPERNODE || (int) $3 <= 0) {
		     sprintf(errmsg, "number of disks per node must be between 0 and  %d", MAX_DISKSPERNODE);
		     yyerror(errmsg);
		   }
		   else 
		     DisksPerNode = (int) $3; 
		 }
		     
             |   CPUS EQUALS NUMBER { 
	           if ($3 <= 0.0)
		     yyerror("number of cpus per node must be > zero");
		   else
		     CPUsPerNode = (int) $3;
		 }

             |   CPUMIPS EQUALS NUMBER {
	           if ($3 <= 0.0)
		     yyerror("MIPS must be > zero");
		   else
		     Mips = $3;
		 }

             |   KBYTES EQUALS NUMBER { 
	           if ($3 <= 0.0)
		     yyerror("node memory must be > zero");
		   else
		     KBytesPerNode = (int) $3;
		 }

             |   PAGESIZE EQUALS NUMBER { 
	           if ($3 <= 0.0)
		     yyerror("page size must be > zero");
		   else
		     PageSize = (int) $3;
		 }

             |   STARTIO_INST EQUALS NUMBER { Inst_StartIO = (int) $3; }

             ;

 NetworkStmt:    NETWORK EQUALS LESSTHAN NetworkAttrList GREATERTHAN SEMI
             |   NETWORK error {yyerror("bad network statement syntax");}
             ;

 NetworkAttrList:   NetworkAttr
             |   NetworkAttrList COMMA NetworkAttr
  
             |   NetworkAttrList error 
                   {yyerror("bad network definition syntax"); }
             ;

 NetworkAttr:    MAXPACKETBYTES EQUALS NUMBER {
		   printf("maxpacketbytes not yet supported.  Ignored\n");
		 }

             |   MAXREQUESTS EQUALS NUMBER {
	           if ($3 <= 0.0)
		     yyerror("max requests must be > zero");
		   else
		     MaxRequests = (int) $3;
		 }
   
             |   GLOBKBYTES EQUALS NUMBER 
                   { GlobKBytes = (int) $3; }

             |   LOCKBYTES EQUALS NUMBER 
                   {LocKBytes = (int) $3; }

             |   MINSENDPROTOCOL EQUALS NUMBER 
                   { MinSendProtocol = (int) $3; }

             |   SENDPROTOCOLPERBYTE EQUALS NUMBER 
                   { SendProtocolPerByte = (int) $3; }

             |   SENDPROTOCOLPERBYTE EQUALS ProbDist
                   { yyerror("piecewise linear costs not yet supported.\n"); }

             |   MINRECVPROTOCOL EQUALS NUMBER 
                   { MinRecvProtocol = (int) $3; }

             |   RECVPROTOCOLPERBYTE EQUALS NUMBER 
                   { RecvProtocolPerByte = (int) $3; }

             |   RECVPROTOCOLPERBYTE EQUALS ProbDist
                   { yyerror("piecewise linear costs not yet supported.\n"); }

             |   MINDELAY EQUALS NUMBER 
                   {MinDelay = $3; }

             |   DELAYPERBYTE EQUALS NUMBER 
                   {DelayPerByte = $3; }

             |   DELAYPERBYTE EQUALS ProbDist
                   { yyerror("piecewise linear costs not yet supported.\n"); }

             |   MINSHORTINSTR EQUALS NUMBER 
                   { MinShortInstr = (int) $3; }

             |   SHORTINSTRPERBYTE EQUALS NUMBER 
                   { ShortInstrPerByte = (int) $3; }

             |   SHORTINSTRPERBYTE EQUALS ProbDist
                   { yyerror("piecewise linear costs not yet supported.\n"); }

             ;

 DiskStmt:       DISKS EQUALS LESSTHAN DiskAttrList GREATERTHAN SEMI
             |   DISKS error {yyerror("bad disks statement syntax");}
             ;

 DiskAttrList:   DiskAttr
             |   DiskAttrList COMMA DiskAttr
  
             |   DiskAttrList error {yyerror("bad disk definition syntax"); }
             ;

 DiskAttr:       PAGESPERCYL EQUALS NUMBER {
                   if ($3 <= 0.0)
		     yyerror("pages per cylinder must be > 0");
		   else
		     PagesPerCyl = (int) $3;
		 }

             |   SEEKFACTOR EQUALS NUMBER { 
                   if ($3 <= 0.0)
		     yyerror("seek factor must be > 0");
		   else
		     SeekFactor = $3; 
		 }

             |   MINLATENCY EQUALS NUMBER { MinLatency = $3; }

             |   ROTATION EQUALS NUMBER { 
                   if ($3 <= 0.0)
		     yyerror("rotation speed must be > 0");
		   else
		     DiskRotation = $3; 
		 }

             |   SETTLE EQUALS NUMBER { SettleTime = $3; }

             |   XFERRATE EQUALS NUMBER { XferRate = $3; }

             |   CACHEPAGES EQUALS NUMBER { DiskCachePages = (int) $3; }

             |   CACHECONTEXTS EQUALS NUMBER { DiskCacheContexts = (int) $3; }
             ;

 Semi:	         SEMI { yyerrok; stmtErr = FALSE; }
             ;
%%

#include "lex.yy.c"

///////////////////////////////////////////////////////////////////////////////
//
//    Name:       yyerror
//  
//    Function:   Does general formatting of all error messages.
//
//    Change Activity:
//
//      10/91 - Original version.  KPB
//
///////////////////////////////////////////////////////////////////////////////

void yyerror(char *s) {
  extern int yynerrs;
  
  yynerrs++;
  stmtErr = TRUE;
  yywhere();
  fputs(s, yyerfp);
  putc('\n', yyerfp);
}

///////////////////////////////////////////////////////////////////////////////
//
//    Name:       yywhere
//  
//    Function:   Prints the location pinpointing part of parser err messages.
//
//   Change Activity:
//
//      10/91 - Original version.  KPB
//
///////////////////////////////////////////////////////////////////////////////

void yywhere() {
  bool colon = FALSE;
  
  // Prints file name part of message

  if (SourceFileName && *SourceFileName && strcmp(SourceFileName, "\"\"")) {
    char *cp=SourceFileName;
    
    int len = strlen(SourceFileName);
    
    if (*cp == '"')
      ++cp, len -= 2;
    if (strncmp(cp, "./", 2) == 0)
      cp += 2, len -= 2;
    fprintf(yyerfp, "file %.*s", len, cp);
    colon = TRUE;
  }

  // Figures out and prints line number part of message

  if (yylineno > 0) {
    if (colon)
      fputs(", ", yyerfp);
    fprintf(yyerfp, "line %d", yylineno - (*yytext == '\n' || ! *yytext));
    colon = TRUE;
  }

  // Prints a nearby token where error occurred, if it can

  if (*yytext) {
    register int i;
    
    for (i=0; i<20; i++)
      if (!yytext[i] || yytext[i] == '\n')
	break;
    if (i) {
      if (colon)
	putc(' ', yyerfp);
      fprintf(yyerfp, "near \"%.*s\"", i, yytext);
      colon = TRUE;
    }
  }

  if (colon)
    fputs(": ", yyerfp);
}

///////////////////////////////////////////////////////////////////////////////
//
//    Name:       yywrap
//  
//    Function:   Gets called by lex when EOF is detected.
//
//    Change Activity:
//
//      10/91 - Original version.  KPB
//
///////////////////////////////////////////////////////////////////////////////

int yywrap() {
  if (Flags[TRACE_PARSER]) {
    printf("\n\nParser symbol table dump:\n-------------------------\n");
    RealSyms.print(4);
    FileSyms.print(4);
    PlanSyms.print(4);
    JobClassSyms.print(4);
    printf("\n");
  }

  return(1); // tell lex to go ahead and quit 
}

///////////////////////////////////////////////////////////////////////////////
//
//    Name:       ResetFile
// 
//    Function:   Sets all accumulated file attributes to invalid values so
//                the next file definition can be checked for missing/invalid
//                attributes.
//
//    Change Activity:
//
//      11/91 - Original version.  KPB
//
///////////////////////////////////////////////////////////////////////////////

void ResetFile() {
  currFileInfo.fileNo = currFileInfo.numTuples =
    currFileInfo.tupleSize = currFileInfo.keySize = 
      currFileInfo.indexType = -1;
  delete currFileInfo.nodeSet;
  delete currFileInfo.diskSet;
  currFileInfo.nodeSet = NULL;
  currFileInfo.diskSet = NULL;
  currFileInfo.indexedFile = NULL;
  currFileInfo.probHit = 0.0;
}


///////////////////////////////////////////////////////////////////////////////
//
//    Name:       ResetPlan
// 
//    Function:   Sets all accumulated plan attributes to invalid values so
//                the next plan definition can be checked for missing/invalid
//                attributes.
//
//    Change Activity:
//
//      8/92 - Original version.  MM
//
///////////////////////////////////////////////////////////////////////////////

void ResetPlan() {
  currPlanInfo.stats = FALSE;
  currPlanInfo.maxmpl = -1;
}

///////////////////////////////////////////////////////////////////////////////
//
//    Name:       CreateFile
// 
//    Function:   Handles a "file =" statement reduction.  Does final error 
//                checks and default handling, creating a real file or index
//                for a random file whose distribution = const_file.
//
//    Change Activity:
//
//      10/91 - Original version.  KPB
//   11/01/91 - Added support for indices.  KPB
//
///////////////////////////////////////////////////////////////////////////////

void CreateFile(char* sym, RandomFile* f, int fileID) {
  char* fileName;
  char  fileDigits[10];
  int   numDisks, numNodes, indexType, fileNum, keySize, i;
  Node** nodeList;
  int* diskList;
  IntegerSet *tempSet = NULL, *tempDiskSet = NULL; // working ptr to a set of integers

  // Builds a file name for error msgs

  if (sym != NULL)                     // named file?
    fileName = sym;
  else {
    sprintf(fileDigits, "%d", fileID);
    fileName = &(fileDigits[0]);
  }

  if (!stmtErr)
    f->type = currFileInfo.type;       // set index or data file

  // Validity checks and determines file/index ID for constant files

  if (!stmtErr && f->dist == const_file) {
    if (sym != NULL) {                 // named file?
      if (currFileInfo.fileNo < 0) {
	sprintf(errmsg, "id must be specified for named files/indices (%s)", 
		fileName);
	yyerror(errmsg);
      }
      else {
	fileNum = currFileInfo.fileNo; // file id is from id= keyword
      }
    }
    else {                             // unnamed file
      if (currFileInfo.fileNo >= 0) {
	sprintf(errmsg, "id cannot be specified for unnamed files/indices (%s)",
		fileName);
	yyerror(errmsg);
      }
      else {
	fileNum = fileID;              // file id is from left side of = sign
      }
    }
	  
    f->fileNo = fileNum;               // set in RandomFile so "notafile" works
  }

  // Handles creation of constant data files

  if (!stmtErr && f->dist == const_file && currFileInfo.type == data_file) { 
					    
    // Checks that required parameters are specified 

    if (currFileInfo.numTuples < 0) {
      sprintf(errmsg, "numtuples must be specified for file %s", fileName);
      yyerror(errmsg);
    }
    
    if (currFileInfo.keySize >= 0) 
      yyerror("keysize parameter doesn't make sense for a data file");

    if (currFileInfo.indexType >= 0)
      yyerror("type parameter doesn't make sense for a data file");

    if (currFileInfo.indexedFile != NULL)
      yyerror("file parameter doesn't make sense for a data file");

    // Verifies/creates array of Node* pointers, if NODE = was specified.
  
    if (currFileInfo.nodeSet != NULL) {
      // if random set, choose the set first before verification of node Ids

      tempSet = new IntegerSet();
      if (currFileInfo.nodeSet->setType == random_intset) {
	tempSet->set = 
	  currFileInfo.nodeSet->randSet->choose(tempSet->card);
      } else {
	tempSet->card = currFileInfo.nodeSet->card;
	tempSet->set = new int [tempSet->card];
	for (i = 0; i < tempSet->card; i++)
	    tempSet->set[i] = currFileInfo.nodeSet->set[i];
      }
      
      for (i = 0; i < tempSet->card; i++) 
	if (notanode(tempSet->set[i])) {
	    printf("invalide node specified %d\n", tempSet->set[i]);
	    break;
	}
    
      // Should really check for duplicate node IDs too !!!!
      
      if (i >= tempSet->card) {
	
	// Translates list of node ids into list of node objects
	
	for (i = 0; i < tempSet->card; i++)
	  tempSet->set[i] = (int) Nodes[tempSet->set[i]];
	
	nodeList = (Node**) tempSet->set;
	numNodes = tempSet->card; 
      } else {
	yyerror("one or more invalid nodes specified");
	delete tempSet;
      }
    } else { 
      
      // If no nodes statement was specified, fully decluster file -> all nodes

      numNodes = NumDiskNodes;
      nodeList = Nodes;
    }
  
    // Verifies/creates list of disk IDs if DISKS = was specified.
        
    if (currFileInfo.diskSet != NULL) {

      tempDiskSet = new IntegerSet();
      if (currFileInfo.diskSet->setType == random_intset)
	tempDiskSet->set = 
	  currFileInfo.diskSet->randSet->choose(tempDiskSet->card);
      else {
	tempDiskSet->card = currFileInfo.diskSet->card;
	tempDiskSet->set = new int [tempDiskSet->card];
	for (i = 0; i < tempDiskSet->card; i++)
	    tempDiskSet->set[i] = currFileInfo.diskSet->set[i];
      }
	
      for (i = 0; i < tempDiskSet->card; i++) 
	if (notadisk(tempDiskSet->set[i])) {
	    printf("invalide node specified %d\n", tempDiskSet->set[i]);
	    break;
	}
    
	for (i = 0; i < tempDiskSet->card; i++) 
	  if (notadisk(tempDiskSet->set[i])) break;
	
	// Should really check for duplicate disk IDs too !!!!
	  
	if (i >= tempDiskSet->card) { 
	  diskList = tempDiskSet->set;
	  numDisks = tempDiskSet->card; //# of disks
	} else {
	  yyerror("one or more invalid disks specified");
	  delete tempDiskSet;
	}
    } else {

      // If no disks statement was specified, fully decluster file -> all disks

      numDisks = DisksPerNode;
      diskList = AllDiskIDs;
    }
    
    // If no errors so far, takes appropriate defaults and cReates a data file
	
    if (!stmtErr) {
      if (currFileInfo.tupleSize < 0) {
	currFileInfo.tupleSize = DEFAULT_TUPSIZE;
      }
      
      f->file = new File(fileNum,
			 numNodes,
			 nodeList,
			 numDisks,
			 diskList,
			 currFileInfo.numTuples,
			 currFileInfo.tupleSize,
			 PERM,
			 currFileInfo.probHit);

      /* Since every constant file must be locatable by number as well as by
	 name, places both the "real" file object AND the corresponding 
	 random file object (distribution = constant) in well known locations
	 which are indexed by file number.  */
      
      ConstFiles[fileNum] = f;              // store in a well known location
      Relations[fileNum] = f->file;         // store in a well known loc

      // Deletes temporary integer set if we needed one for a random node
      // assignment.

      delete tempSet; // NOP if = NULL
    }
  } // constant data file
    
  // Handles creation of constant index files

  if (!stmtErr && f->dist == const_file && currFileInfo.type == index_file) { 
					    
    // Checks that required parameters are specified

    if (currFileInfo.indexedFile == NULL) {
      sprintf(errmsg, "file must be specified for index %s", fileName);
      yyerror(errmsg);
    }
    
    if (currFileInfo.nodeSet != NULL)
      yyerror("nodes parameter doesn't make sense for an index file");

    if (currFileInfo.diskSet != NULL)
      yyerror("disks parameter doesn't make sense for an index file");

    if (currFileInfo.tupleSize >= 0)
      yyerror("tuplesize parameter doesn't make sense for an index file");

    if (currFileInfo.numTuples >= 0)
      yyerror("numtuples parameter doesn't make sense for an index file");

    // If no errors so far, takes appropriate defaults and cReates an index
	
    if (!stmtErr) {
      if (currFileInfo.indexType < 0) 
	indexType = UNCLUSTERED;
      else
	indexType = currFileInfo.indexType;
      
      if (currFileInfo.keySize < 0) 
	keySize = 4;
      else
	keySize = currFileInfo.keySize;
    
      f->index = new Index(fileNum,
			  currFileInfo.indexedFile->file,
			  keySize,
			  indexType,
	   		  currFileInfo.probHit);
      
      /* Since every constant file must be locatable by number as well as by
	 name, places both the "real" file object AND the corresponding 
	 random file object (distribution = constant) in well known locations
	 which are indexed by file number.  */
      
      ConstFiles[fileNum] = f;            // store in a well known location
      Indices[fileNum] = f->index;        // store in a well known loc
    }
  } // constant index file
    
  /* If no previous errors, and this is a named file, attempts to add the 
     name/object pair to the file symbol table, backing out everything if
     some error occurs adding the symbol. */

  // want to make up symbols for unnamed files??

  if (!stmtErr && sym != NULL) {
    if (!FileSyms.add(sym, f)) {          // if some error adding symbol
      if (f->dist == const_file && f->type == data_file) { // const data file?
        ConstFiles[f->fileNo] = NULL;
	Relations[f->fileNo] = NULL;
        delete f->file;                   // delete real data file 
      }

      if (f->dist == const_file && f->type == index_file) { //const index file?
        ConstFiles[f->fileNo] = NULL;
	Indices[f->fileNo] = NULL;
        delete f->index;                  // delete real index file 
      }

      delete f;                           // delete random file
    }
  }
	
  stmtErr = FALSE;                        // clear err indicator
}

///////////////////////////////////////////////////////////////////////////////
//
//    Name:       CreateJobClass
// 
//    Function:   Handles a "class =" statement reduction, does final error
//                checks and defaulting, and if OK, adds the job class 
//                definition to the symbol table.
//
//    Change Activity:
//
//      10/91 - Original version.  KPB
//
///////////////////////////////////////////////////////////////////////////////

void CreateJobClass(char* sym, JobClass* jc) {

  // Performs semantic error checks that can only be done when the entire 
  // class statement has been recognized.

  if (jc->interArrivalSecs != NULL) {
    if ((jc->population != -1) || 
	(jc->thinksecs != NULL)) {
      sprintf(errmsg, "interarrivalsecs is mutually exclusive with thinksecs and population (%s)", sym);
      yyerror(errmsg);
    }
    
    if (jc->maxMPL < 0) {
      sprintf(errmsg, "maxmpl must be specified for open classes (%s) to prevent possible saturation", sym);
      yyerror(errmsg);
    }      
  }

  if (jc->plans == NULL) {
    sprintf(errmsg, "no plan distribution for job class %s", sym);
    yyerror(errmsg);
  }

  // if plan distribution is random, primary class needed to stop simulation
  if ((jc->plans != NULL) && 
      (jc->plans->dist == rand_dist) && (jc->population > 0))
      PrimaryClassNeeded = TRUE;

  if ((jc->plans != NULL) && jc->stopCount >= 0) { // primary class
      if (jc->plans->dist == determ_dist) {
	  PrimaryClassFound = TRUE;
      } else if (jc->maxCompletions <= 0) {
	  sprintf(errmsg, "ERROR: primary class (%s) cannot have a random plan distribution", sym);
	  yyerror(errmsg);
      } else {
	  PrimaryClassFound = TRUE;
      }
  } else if (jc->maxCompletions > 0) {
      sprintf(errmsg, "ERROR: cannot have maxcompletions for non-primary class (%s)", sym);
      yyerror(errmsg);
  }

  // If statwaitsecs was specified, must call a method in job class to update
  // all the tables with the new delay time.

  if (jc->statWaitSecs != 0.0) jc->updateDelay();

  // Fills in any defaults needed.

  if (!stmtErr) {
    if (jc->interArrivalSecs == NULL) {

      if (jc->population < 0) {
	jc->population = 1;
      }
      
      if (jc->maxMPL < 0) {
	jc->maxMPL = INT_MAX;
      }

      if (jc->thinksecs == NULL) {
	jc->thinksecs = new RandomReal;
	jc->thinksecs->dist = const_real;
	jc->thinksecs->val = 0.0;
      }

    } else {
      jc->population = jc->maxMPL;  // required by sim.c to set CSIM limits
    }
  }

  // Attempts to add this job class into the symbol table and update stats.

  if (stmtErr)               // if previous err with this class definition
    delete jc;
  else
    if (!JobClassSyms.add(sym, jc))
      delete jc;             // some error adding symbol
    else {                   // everything ok, job class is defined
      jc->name = sym;        // put symbolic name inside jobclass object
      JobClasses[NumClasses++] = jc; // stuff in well known location
      jc->id = NumClasses - 1; // store its index 
    }

  stmtErr = FALSE;
}

///////////////////////////////////////////////////////////////////////////////
//
//    Name:       CreatePlan
// 
//    Function:   Handles a "plan =" statement reduction.
//
//    Change Activity:
//
//      10/91 - Original version.  KPB
//
///////////////////////////////////////////////////////////////////////////////

void CreatePlan(char* sym, RandomPlan* p) {
  
  if (!stmtErr) {

    // Determines type of scheduling based on plan naming convention - KLUDGE!

    for (i = 0; i < MAX_SCHEDTYPES; i++)
      if (strncmp(sym, PlanPrefixes[i], strlen(PlanPrefixes[i])) == 0) {
	p->schedType = i;
	break;
      }
    
    // Default to left deep if no recognizable plan prefix is present

    if (i >= MAX_SCHEDTYPES)
      p->schedType = LEFTDEEP;

    // Unless the plan is already named, saves the symbolic name for this 
    // plan inside itself for debugging.  Plans record their first given name.
    // This MUST be performed before calling the activateStats() method, since
    // it must know which plans are named versus unnamed!!!!

    if (p->name == NULL) p->name = sym;

    // If statistics are desired for this plan name, sets up to do so.
    if (currPlanInfo.stats) p->activateStats(p);

    // set the maxmpl for the plan if any
    if (currPlanInfo.maxmpl < 0) {
          p->maxMPL = INT_MAX;
    } else {
          p->maxMPL = currPlanInfo.maxmpl;
    }

    if (!PlanSyms.add(sym, p))
      delete p;                // some error adding symbol
  }
  stmtErr = FALSE;
}

