digit           [0-9]
letter		[a-zA-Z]
letter_or_digit [a-zA-Z_0-9]
white_space     [ \t\n]
other           [^0-9a-zA-Z\t\n ]
%s comment
%p 6000
%e 1500
%n 1000

%{
extern "C" {
  double atof(const char *s);
};
%}

%%
[ \t]+		;
\n		;

"="		return(EQUALS);
".."		return(DOTDOT);
","		return(COMMA);
";"		return(SEMI);
"{"		return(LBRACE);
"}"		return(RBRACE);
"["		return(LBRACK);
"]"		return(RBRACK);
"("		return(LPAREN);
")"		return(RPAREN);
"<"             return(LESSTHAN);
">"             return(GREATERTHAN);
"/"             return(SLASH);
"*"             return(SPLAT);

real            return(REAL);
plan            return(PLAN);
file            return(FILEKW);
index           return(INDEXKW);
class           return(JOBCLASS);

id              return(ID);
tuplesize       return(TUPLESIZE);
numtuples       return(NUMTUPLES);
disks           return(DISKS);
nodes           return(NODES);
type            return(TYPE);
clustered       return(CLUSTEREDKW);
unclustered     return(UNCLUSTEREDKW);
keysize         return(KEYSIZE);
probhit         return(PROBHIT);
thinksecs       return(THINKSECS);
waitsecs        return(WAITSECS);
statwaitsecs    return(STATWAITSECS);
interarrivalsecs return(INTERARRIVALSECS);
population      return(POPULATION);
primary         return(PRIMARY);
maxcompletions  return(MAXCOMPLETIONS);
plans           return(PLANS);
cpus            return(CPUS);
cpumips         return(CPUMIPS);
kbytes          return(KBYTES);
"pages/cyl"     return(PAGESPERCYL);
pagesize        return(PAGESIZE);
seekfactor      return(SEEKFACTOR);
minlatency      return(MINLATENCY);
rotation        return(ROTATION);
settle          return(SETTLE);
xferrate        return(XFERRATE);     
cachepages      return(CACHEPAGES);
cachecontexts   return(CACHECONTEXTS);
bpool           return(BPOOL);
min             return(MIN);
max             return(MAX);
bytes           return(BYTES);
kbytes          return(KBYTES);
mbytes          return(MBYTES);
maxmpl          return(MAXMPL);
stats           return(STATS);
inst_startio    return(STARTIO_INST);
network         return(NETWORK);
maxpacketbytes  return(MAXPACKETBYTES);
maxrequests     return(MAXREQUESTS);
globpoolKbytes |
globpoolKBytes |
globpoolkbytes      return(GLOBKBYTES);
locpoolKbytes |
locpoolKBytes |
locpoolkbytes       return(LOCKBYTES);
minsendprotocol     return(MINSENDPROTOCOL);
"sendprotocol/byte" return(SENDPROTOCOLPERBYTE);
minrecvprotocol     return(MINRECVPROTOCOL);
"recvprotocol/byte" return(RECVPROTOCOLPERBYTE);
mindelay        return(MINDELAY);
"delay/byte"    return(DELAYPERBYTE);
minshortinstr   return(MINSHORTINSTR);
"shortinstr/byte" return(SHORTINSTRPERBYTE);


filescan |
fscan           return(FILESCAN);
indexscan |
iscan           return(INDEXSCAN);
proj            return(PROJ);
store           return(STOREOP);    /* STORE is a #defined operator type */
unique          return(UNIQUE);
sort            return(SORT);
sortunique      return(SORTUNIQUE);
aggr            return(AGGR);
hashjoin  |
hjoin           return(HASHJOIN);
xprod           return(XPROD);
update          return(UPDATE);
where           return(WHERE);
union           return(UNION);
intsect |
intersection    return(INTERSECTION);
diff    |
difference      return(DIFFERENCE);
termsend        return(TERMSEND);

sites           return(SITES);
expntl  |
exponential     return(EXPNTL);
normal          return(NORMAL);
uniform         return(UNIFORM);
random          return(RANDOM);

"//".*\n	;
"/*"            BEGIN(comment);
<comment>[^*]   ;
<comment>"*"+[^/]* ;
<comment>"*"+"/"   BEGIN(INITIAL);

{letter}{letter_or_digit}* 	{
		yylval.string = new char[strlen(yytext)+1];
		strcpy(yylval.string,yytext);
		return(NAME);
		}

{digit}*"."{digit}+     |
{digit}+	{
		yylval.floatval = (float)atof(yytext);
		return(NUMBER);
		}

{other}         return(yytext[0]);  

%%
