
# line 2 "parser.y"
#define isparser "yup"
#include "string.h"
#include "dbsim.h"
#include "object.h"
#include "node.h"
#include "query.h"
#include "language.h"
#include "disk.h"
#include "db.h"
#include "cost.h"
#include <stdio.h>
#include <math.h>
#include <limits.h>

///////////////////////////////////////////////////////////////////////////////
//
//    Name:       parser.y (yyparse, yyerror, yywhere, yywrap)
//  
//    Function:   A lex/yacc generated parser which translates hardware and 
//                software configuration, database definition, and workload
//                definition statements into "template objects" (see 
//                language.h).  These template objects are then used to cReate 
//                specific instances of simulator objects:  nodes, disks, 
//                terminals, and queries.
//
//    Change Activity:
//
//     09/91 - Original version.  KPB
//  11/01/91 - Added index support.  KPB
//
///////////////////////////////////////////////////////////////////////////////

extern SymTable   RealSyms;      // real num symbol table
extern SymTable   FileSyms;      // file symbol table
extern SymTable   PlanSyms;      // plan symbol table
extern SymTable   JobClassSyms;  // job class symbol table
extern Node       *Nodes[MAX_NODES];
extern Index      *Indices[MAX_FILES];
extern File       *Relations[MAX_FILES];
extern JobClass   *JobClasses[MAX_CLASSES];
extern bool       OpenClasses;   // one or more open classes exist
extern int*       AllDiskIDs;    // array of one integer per DisksPerNode
extern bool       PrimaryClassNeeded, PrimaryClassFound;
extern int        Inst_StartIO;

class IntegerSet;

ProbDist   *currProbDist; // working ptr to a user specified prob distr array
ProbDist   *currProbDist2;// working ptr to a user specified prob distr array
RandomReal *currRealNum;  // working ptr to a random real number 
RandomFile *currFile;     // working ptr to a random file 
RandomPlan *currPlan;     // working ptr to a random plan 
JobClass   *currJClass;   // working ptr to a job class 
IntegerSet *currIntSet;   // working ptr to a set of integers
MemReq     *currMemReq;   // working ptr to a memory requirement
int        i;
bool       stmtErr = FALSE; // some syntax/semantic error on current stmt

static char errmsg[80];
extern char yytext[];     // lex managed, holds current input file line
extern int yyleng;
extern int yylineno;
FILE*      yyerfp = stdout;
extern char *SourceFileName;

void yyerror(char* s);
void       yywhere();
int        yylook();
int        yyback(int*, int);
int        yywrap();
void       CreateFile(char* sym, RandomFile* f, int fileID);
void       ResetFile();
void       CreateJobClass(char* sym, JobClass* jc);
void       CreatePlan(char* sym, RandomPlan* p);

struct planInfo {         // place to accumulate plan attributes during parse
  bool stats;
  int  maxmpl;
} currPlanInfo = {FALSE, -1};

struct fileinfo {         // place to accumulate file attributes during parse
  fileType type;          // index_file or data_file
  int      fileNo;
  IntegerSet* nodeSet;
  IntegerSet* diskSet;
  int      tupleSize;
  int      numTuples;
  int      keySize;
  int      indexType;
  float    probHit;
  RandomFile* indexedFile; 
} currFileInfo =  {data_file, -1, NULL, NULL, -1, -1, -1, -1, 0.0, NULL};

typedef enum intset_type {random_intset, constant_intset};

class IntegerSet {          // random or fixed set of integers
  public:			    
    intset_type setType;    // random or constant			      
    int      max;           // max cardinality and element value allowed
    int      card;          // current cardinality (number of elements)
    int*     set;           // actual set (array) of integers
    RandomSet* randSet;     // points to RandomSet if specified that way

    IntegerSet() {

      // Prepare for default of constant_intset and max unknown

      setType = constant_intset;
      max = 0;  
      card = 0;  
      set = NULL;
      randSet = NULL;
    };

    IntegerSet(int mx) {

      // Prepare for default of constant_intset

      setType = constant_intset;
      max = mx;  
      card = 0;  
      set = new int[max];
      randSet = NULL;
    };

    ~IntegerSet() {delete set;};

    int add(int e) {
      if ((int) e < max && card < max) {
	set[card++] = (int) e;
	return(0);          // indicate successful
      }
      else {
	sprintf(errmsg, "max cardinality is %d and max element value is %d for this set", max, max-1);
	yyerror(errmsg);
	return(-1);         // indicate some problem
      }
    };
    
    // add the sites on which file declustered to the integer set
    int add(File* file) {
        int i;
	for (i = 0; i < file->numNodes; i++) {
	  this->add(file->storageNode(i)->id);
	}
	return(0);          // indicate successful
    }
};


# line 155 "parser.y"
typedef union  {               
        int intval;
	float floatval;
	bool boolval;
	char *string;
	scanTypes stype;
        ProbDist* probdist;
        RandomReal* randreal;
        RandomFile* randfile;
	RandomPlan* randplan;
        JobClass* jclass;
	IntegerSet* iset;
	MemReq* mreq;
        } YYSTYPE;
# define FILENAME 257
# define NAME 258
# define NUMBER 259
# define SEMI 260
# define COMMA 261
# define EQUALS 262
# define SLASH 263
# define DOTDOT 264
# define SPLAT 265
# define LPAREN 266
# define RPAREN 267
# define LBRACK 268
# define RBRACK 269
# define LBRACE 270
# define RBRACE 271
# define LESSTHAN 272
# define GREATERTHAN 273
# define REAL 274
# define PLAN 275
# define FILEKW 276
# define INDEXKW 277
# define JOBCLASS 278
# define INTERARRIVALSECS 279
# define PROJ 280
# define SORT 281
# define AGGR 282
# define XPROD 283
# define UPDATE 284
# define WHERE 285
# define INDEXSCAN 286
# define FILESCAN 287
# define STOREOP 288
# define HASHJOIN 289
# define INDEXREPLICATEJOIN 290
# define INDEXREPLICATEJOINP 291
# define UNION 292
# define INTERSECTION 293
# define DIFFERENCE 294
# define TERMSEND 295
# define UNIQUE 296
# define SORTUNIQUE 297
# define EXPNTL 298
# define NORMAL 299
# define UNIFORM 300
# define RANDOM 301
# define SITES 302
# define ID 303
# define TUPLESIZE 304
# define NUMTUPLES 305
# define DISKS 306
# define NODES 307
# define KEYSIZE 308
# define TYPE 309
# define CLUSTEREDKW 310
# define PROBHIT 311
# define UNCLUSTEREDKW 312
# define THINKSECS 313
# define POPULATION 314
# define PLANS 315
# define CPUS 316
# define CPUMIPS 317
# define KBYTES 318
# define PAGESIZE 319
# define PAGESPERCYL 320
# define SEEKFACTOR 321
# define MINLATENCY 322
# define ROTATION 323
# define SETTLE 324
# define XFERRATE 325
# define PRIMARY 326
# define CACHEPAGES 327
# define BPOOL 328
# define MIN 329
# define MAX 330
# define BYTES 331
# define MBYTES 332
# define MAXMPL 333
# define STATS 334
# define CACHECONTEXTS 335
# define STARTIO_INST 336
# define NETWORK 337
# define MAXPACKETBYTES 338
# define MAXREQUESTS 339
# define GLOBKBYTES 340
# define LOCKBYTES 341
# define MINSENDPROTOCOL 342
# define SENDPROTOCOLPERBYTE 343
# define MINRECVPROTOCOL 344
# define RECVPROTOCOLPERBYTE 345
# define MINDELAY 346
# define DELAYPERBYTE 347
# define MINSHORTINSTR 348
# define SHORTINSTRPERBYTE 349
# define WAITSECS 350
# define STATWAITSECS 351
# define MAXCOMPLETIONS 352
#define yyclearin yychar = -1
#define yyerrok yyerrflag = 0
extern int yychar;
extern short yyerrflag;
#ifndef YYMAXDEPTH
#define YYMAXDEPTH 150
#endif
YYSTYPE yylval, yyval;
# define YYERRCODE 256

# line 1315 "parser.y"


#include "lex.yy.c"

///////////////////////////////////////////////////////////////////////////////
//
//    Name:       yyerror
//  
//    Function:   Does general formatting of all error messages.
//
//    Change Activity:
//
//      10/91 - Original version.  KPB
//
///////////////////////////////////////////////////////////////////////////////

void yyerror(char *s) {
  extern int yynerrs;
  
  yynerrs++;
  stmtErr = TRUE;
  yywhere();
  fputs(s, yyerfp);
  putc('\n', yyerfp);
}

///////////////////////////////////////////////////////////////////////////////
//
//    Name:       yywhere
//  
//    Function:   Prints the location pinpointing part of parser err messages.
//
//   Change Activity:
//
//      10/91 - Original version.  KPB
//
///////////////////////////////////////////////////////////////////////////////

void yywhere() {
  bool colon = FALSE;
  
  // Prints file name part of message

  if (SourceFileName && *SourceFileName && strcmp(SourceFileName, "\"\"")) {
    char *cp=SourceFileName;
    
    int len = strlen(SourceFileName);
    
    if (*cp == '"')
      ++cp, len -= 2;
    if (strncmp(cp, "./", 2) == 0)
      cp += 2, len -= 2;
    fprintf(yyerfp, "file %.*s", len, cp);
    colon = TRUE;
  }

  // Figures out and prints line number part of message

  if (yylineno > 0) {
    if (colon)
      fputs(", ", yyerfp);
    fprintf(yyerfp, "line %d", yylineno - (*yytext == '\n' || ! *yytext));
    colon = TRUE;
  }

  // Prints a nearby token where error occurred, if it can

  if (*yytext) {
    register int i;
    
    for (i=0; i<20; i++)
      if (!yytext[i] || yytext[i] == '\n')
	break;
    if (i) {
      if (colon)
	putc(' ', yyerfp);
      fprintf(yyerfp, "near \"%.*s\"", i, yytext);
      colon = TRUE;
    }
  }

  if (colon)
    fputs(": ", yyerfp);
}

///////////////////////////////////////////////////////////////////////////////
//
//    Name:       yywrap
//  
//    Function:   Gets called by lex when EOF is detected.
//
//    Change Activity:
//
//      10/91 - Original version.  KPB
//
///////////////////////////////////////////////////////////////////////////////

int yywrap() {
  if (Flags[TRACE_PARSER]) {
    printf("\n\nParser symbol table dump:\n-------------------------\n");
    RealSyms.print(4);
    FileSyms.print(4);
    PlanSyms.print(4);
    JobClassSyms.print(4);
    printf("\n");
  }

  return(1); // tell lex to go ahead and quit 
}

///////////////////////////////////////////////////////////////////////////////
//
//    Name:       ResetFile
// 
//    Function:   Sets all accumulated file attributes to invalid values so
//                the next file definition can be checked for missing/invalid
//                attributes.
//
//    Change Activity:
//
//      11/91 - Original version.  KPB
//
///////////////////////////////////////////////////////////////////////////////

void ResetFile() {
  currFileInfo.fileNo = currFileInfo.numTuples =
    currFileInfo.tupleSize = currFileInfo.keySize = 
      currFileInfo.indexType = -1;
  delete currFileInfo.nodeSet;
  delete currFileInfo.diskSet;
  currFileInfo.nodeSet = NULL;
  currFileInfo.diskSet = NULL;
  currFileInfo.indexedFile = NULL;
  currFileInfo.probHit = 0.0;
}


///////////////////////////////////////////////////////////////////////////////
//
//    Name:       ResetPlan
// 
//    Function:   Sets all accumulated plan attributes to invalid values so
//                the next plan definition can be checked for missing/invalid
//                attributes.
//
//    Change Activity:
//
//      8/92 - Original version.  MM
//
///////////////////////////////////////////////////////////////////////////////

void ResetPlan() {
  currPlanInfo.stats = FALSE;
  currPlanInfo.maxmpl = -1;
}

///////////////////////////////////////////////////////////////////////////////
//
//    Name:       CreateFile
// 
//    Function:   Handles a "file =" statement reduction.  Does final error 
//                checks and default handling, creating a real file or index
//                for a random file whose distribution = const_file.
//
//    Change Activity:
//
//      10/91 - Original version.  KPB
//   11/01/91 - Added support for indices.  KPB
//
///////////////////////////////////////////////////////////////////////////////

void CreateFile(char* sym, RandomFile* f, int fileID) {
  char* fileName;
  char  fileDigits[10];
  int   numDisks, numNodes, indexType, fileNum, keySize, i;
  Node** nodeList;
  int* diskList;
  IntegerSet *tempSet = NULL, *tempDiskSet = NULL; // working ptr to a set of integers

  // Builds a file name for error msgs

  if (sym != NULL)                     // named file?
    fileName = sym;
  else {
    sprintf(fileDigits, "%d", fileID);
    fileName = &(fileDigits[0]);
  }

  if (!stmtErr)
    f->type = currFileInfo.type;       // set index or data file

  // Validity checks and determines file/index ID for constant files

  if (!stmtErr && f->dist == const_file) {
    if (sym != NULL) {                 // named file?
      if (currFileInfo.fileNo < 0) {
	sprintf(errmsg, "id must be specified for named files/indices (%s)", 
		fileName);
	yyerror(errmsg);
      }
      else {
	fileNum = currFileInfo.fileNo; // file id is from id= keyword
      }
    }
    else {                             // unnamed file
      if (currFileInfo.fileNo >= 0) {
	sprintf(errmsg, "id cannot be specified for unnamed files/indices (%s)",
		fileName);
	yyerror(errmsg);
      }
      else {
	fileNum = fileID;              // file id is from left side of = sign
      }
    }
	  
    f->fileNo = fileNum;               // set in RandomFile so "notafile" works
  }

  // Handles creation of constant data files

  if (!stmtErr && f->dist == const_file && currFileInfo.type == data_file) { 
					    
    // Checks that required parameters are specified 

    if (currFileInfo.numTuples < 0) {
      sprintf(errmsg, "numtuples must be specified for file %s", fileName);
      yyerror(errmsg);
    }
    
    if (currFileInfo.keySize >= 0) 
      yyerror("keysize parameter doesn't make sense for a data file");

    if (currFileInfo.indexType >= 0)
      yyerror("type parameter doesn't make sense for a data file");

    if (currFileInfo.indexedFile != NULL)
      yyerror("file parameter doesn't make sense for a data file");

    // Verifies/creates array of Node* pointers, if NODE = was specified.
  
    if (currFileInfo.nodeSet != NULL) {
      // if random set, choose the set first before verification of node Ids

      tempSet = new IntegerSet();
      if (currFileInfo.nodeSet->setType == random_intset) {
	tempSet->set = 
	  currFileInfo.nodeSet->randSet->choose(tempSet->card);
      } else {
	tempSet->card = currFileInfo.nodeSet->card;
	tempSet->set = new int [tempSet->card];
	for (i = 0; i < tempSet->card; i++)
	    tempSet->set[i] = currFileInfo.nodeSet->set[i];
      }
      
      for (i = 0; i < tempSet->card; i++) 
	if (notanode(tempSet->set[i])) {
	    printf("invalide node specified %d\n", tempSet->set[i]);
	    break;
	}
    
      // Should really check for duplicate node IDs too !!!!
      
      if (i >= tempSet->card) {
	
	// Translates list of node ids into list of node objects
	
	for (i = 0; i < tempSet->card; i++)
	  tempSet->set[i] = (int) Nodes[tempSet->set[i]];
	
	nodeList = (Node**) tempSet->set;
	numNodes = tempSet->card; 
      } else {
	yyerror("one or more invalid nodes specified");
	delete tempSet;
      }
    } else { 
      
      // If no nodes statement was specified, fully decluster file -> all nodes

      numNodes = NumDiskNodes;
      nodeList = Nodes;
    }
  
    // Verifies/creates list of disk IDs if DISKS = was specified.
        
    if (currFileInfo.diskSet != NULL) {

      tempDiskSet = new IntegerSet();
      if (currFileInfo.diskSet->setType == random_intset)
	tempDiskSet->set = 
	  currFileInfo.diskSet->randSet->choose(tempDiskSet->card);
      else {
	tempDiskSet->card = currFileInfo.diskSet->card;
	tempDiskSet->set = new int [tempDiskSet->card];
	for (i = 0; i < tempDiskSet->card; i++)
	    tempDiskSet->set[i] = currFileInfo.diskSet->set[i];
      }
	
      for (i = 0; i < tempDiskSet->card; i++) 
	if (notadisk(tempDiskSet->set[i])) {
	    printf("invalide node specified %d\n", tempDiskSet->set[i]);
	    break;
	}
    
	for (i = 0; i < tempDiskSet->card; i++) 
	  if (notadisk(tempDiskSet->set[i])) break;
	
	// Should really check for duplicate disk IDs too !!!!
	  
	if (i >= tempDiskSet->card) { 
	  diskList = tempDiskSet->set;
	  numDisks = tempDiskSet->card; //# of disks
	} else {
	  yyerror("one or more invalid disks specified");
	  delete tempDiskSet;
	}
    } else {

      // If no disks statement was specified, fully decluster file -> all disks

      numDisks = DisksPerNode;
      diskList = AllDiskIDs;
    }
    
    // If no errors so far, takes appropriate defaults and cReates a data file
	
    if (!stmtErr) {
      if (currFileInfo.tupleSize < 0) {
	currFileInfo.tupleSize = DEFAULT_TUPSIZE;
      }
      
      f->file = new File(fileNum,
			 numNodes,
			 nodeList,
			 numDisks,
			 diskList,
			 currFileInfo.numTuples,
			 currFileInfo.tupleSize,
			 PERM,
			 currFileInfo.probHit);

      /* Since every constant file must be locatable by number as well as by
	 name, places both the "real" file object AND the corresponding 
	 random file object (distribution = constant) in well known locations
	 which are indexed by file number.  */
      
      ConstFiles[fileNum] = f;              // store in a well known location
      Relations[fileNum] = f->file;         // store in a well known loc

      // Deletes temporary integer set if we needed one for a random node
      // assignment.

      delete tempSet; // NOP if = NULL
    }
  } // constant data file
    
  // Handles creation of constant index files

  if (!stmtErr && f->dist == const_file && currFileInfo.type == index_file) { 
					    
    // Checks that required parameters are specified

    if (currFileInfo.indexedFile == NULL) {
      sprintf(errmsg, "file must be specified for index %s", fileName);
      yyerror(errmsg);
    }
    
    if (currFileInfo.nodeSet != NULL)
      yyerror("nodes parameter doesn't make sense for an index file");

    if (currFileInfo.diskSet != NULL)
      yyerror("disks parameter doesn't make sense for an index file");

    if (currFileInfo.tupleSize >= 0)
      yyerror("tuplesize parameter doesn't make sense for an index file");

    if (currFileInfo.numTuples >= 0)
      yyerror("numtuples parameter doesn't make sense for an index file");

    // If no errors so far, takes appropriate defaults and cReates an index
	
    if (!stmtErr) {
      if (currFileInfo.indexType < 0) 
	indexType = UNCLUSTERED;
      else
	indexType = currFileInfo.indexType;
      
      if (currFileInfo.keySize < 0) 
	keySize = 4;
      else
	keySize = currFileInfo.keySize;
    
      f->index = new Index(fileNum,
			  currFileInfo.indexedFile->file,
			  keySize,
			  indexType,
	   		  currFileInfo.probHit);
      
      /* Since every constant file must be locatable by number as well as by
	 name, places both the "real" file object AND the corresponding 
	 random file object (distribution = constant) in well known locations
	 which are indexed by file number.  */
      
      ConstFiles[fileNum] = f;            // store in a well known location
      Indices[fileNum] = f->index;        // store in a well known loc
    }
  } // constant index file
    
  /* If no previous errors, and this is a named file, attempts to add the 
     name/object pair to the file symbol table, backing out everything if
     some error occurs adding the symbol. */

  // want to make up symbols for unnamed files??

  if (!stmtErr && sym != NULL) {
    if (!FileSyms.add(sym, f)) {          // if some error adding symbol
      if (f->dist == const_file && f->type == data_file) { // const data file?
        ConstFiles[f->fileNo] = NULL;
	Relations[f->fileNo] = NULL;
        delete f->file;                   // delete real data file 
      }

      if (f->dist == const_file && f->type == index_file) { //const index file?
        ConstFiles[f->fileNo] = NULL;
	Indices[f->fileNo] = NULL;
        delete f->index;                  // delete real index file 
      }

      delete f;                           // delete random file
    }
  }
	
  stmtErr = FALSE;                        // clear err indicator
}

///////////////////////////////////////////////////////////////////////////////
//
//    Name:       CreateJobClass
// 
//    Function:   Handles a "class =" statement reduction, does final error
//                checks and defaulting, and if OK, adds the job class 
//                definition to the symbol table.
//
//    Change Activity:
//
//      10/91 - Original version.  KPB
//
///////////////////////////////////////////////////////////////////////////////

void CreateJobClass(char* sym, JobClass* jc) {

  // Performs semantic error checks that can only be done when the entire 
  // class statement has been recognized.

  if (jc->interArrivalSecs != NULL) {
    if ((jc->population != -1) || 
	(jc->thinksecs != NULL)) {
      sprintf(errmsg, "interarrivalsecs is mutually exclusive with thinksecs and population (%s)", sym);
      yyerror(errmsg);
    }
    
    if (jc->maxMPL < 0) {
      sprintf(errmsg, "maxmpl must be specified for open classes (%s) to prevent possible saturation", sym);
      yyerror(errmsg);
    }      
  }

  if (jc->plans == NULL) {
    sprintf(errmsg, "no plan distribution for job class %s", sym);
    yyerror(errmsg);
  }

  // if plan distribution is random, primary class needed to stop simulation
  if ((jc->plans != NULL) && 
      (jc->plans->dist == rand_dist) && (jc->population > 0))
      PrimaryClassNeeded = TRUE;

  if ((jc->plans != NULL) && jc->stopCount >= 0) { // primary class
      if (jc->plans->dist == determ_dist) {
	  PrimaryClassFound = TRUE;
      } else if (jc->maxCompletions <= 0) {
	  sprintf(errmsg, "ERROR: primary class (%s) cannot have a random plan distribution", sym);
	  yyerror(errmsg);
      } else {
	  PrimaryClassFound = TRUE;
      }
  } else if (jc->maxCompletions > 0) {
      sprintf(errmsg, "ERROR: cannot have maxcompletions for non-primary class (%s)", sym);
      yyerror(errmsg);
  }

  // If statwaitsecs was specified, must call a method in job class to update
  // all the tables with the new delay time.

  if (jc->statWaitSecs != 0.0) jc->updateDelay();

  // Fills in any defaults needed.

  if (!stmtErr) {
    if (jc->interArrivalSecs == NULL) {

      if (jc->population < 0) {
	jc->population = 1;
      }
      
      if (jc->maxMPL < 0) {
	jc->maxMPL = INT_MAX;
      }

      if (jc->thinksecs == NULL) {
	jc->thinksecs = new RandomReal;
	jc->thinksecs->dist = const_real;
	jc->thinksecs->val = 0.0;
      }

    } else {
      jc->population = jc->maxMPL;  // required by sim.c to set CSIM limits
    }
  }

  // Attempts to add this job class into the symbol table and update stats.

  if (stmtErr)               // if previous err with this class definition
    delete jc;
  else
    if (!JobClassSyms.add(sym, jc))
      delete jc;             // some error adding symbol
    else {                   // everything ok, job class is defined
      jc->name = sym;        // put symbolic name inside jobclass object
      JobClasses[NumClasses++] = jc; // stuff in well known location
      jc->id = NumClasses - 1; // store its index 
    }

  stmtErr = FALSE;
}

///////////////////////////////////////////////////////////////////////////////
//
//    Name:       CreatePlan
// 
//    Function:   Handles a "plan =" statement reduction.
//
//    Change Activity:
//
//      10/91 - Original version.  KPB
//
///////////////////////////////////////////////////////////////////////////////

void CreatePlan(char* sym, RandomPlan* p) {
  
  if (!stmtErr) {

    // Determines type of scheduling based on plan naming convention - KLUDGE!

    for (i = 0; i < MAX_SCHEDTYPES; i++)
      if (strncmp(sym, PlanPrefixes[i], strlen(PlanPrefixes[i])) == 0) {
	p->schedType = i;
	break;
      }
    
    // Default to left deep if no recognizable plan prefix is present

    if (i >= MAX_SCHEDTYPES)
      p->schedType = LEFTDEEP;

    // Unless the plan is already named, saves the symbolic name for this 
    // plan inside itself for debugging.  Plans record their first given name.
    // This MUST be performed before calling the activateStats() method, since
    // it must know which plans are named versus unnamed!!!!

    if (p->name == NULL) p->name = sym;

    // If statistics are desired for this plan name, sets up to do so.
    if (currPlanInfo.stats) p->activateStats(p);

    // set the maxmpl for the plan if any
    if (currPlanInfo.maxmpl < 0) {
          p->maxMPL = INT_MAX;
    } else {
          p->maxMPL = currPlanInfo.maxmpl;
    }

    if (!PlanSyms.add(sym, p))
      delete p;                // some error adding symbol
  }
  stmtErr = FALSE;
}

short yyexca[] ={
-1, 1,
	0, -1,
	-2, 0,
-1, 72,
	302, 15,
	-2, 11,
-1, 399,
	265, 29,
	-2, 69,
	};
# define YYNPROD 214
# define YYLAST 772
short yyact[]={

 400,  71, 398, 419, 239, 218, 231,  88,  87, 424,
 211,  10, 214, 131, 100, 101, 102, 103, 104, 105,
 106, 107, 108, 109, 110, 111, 423, 133, 132,  11,
  15,  12,  13,  14, 381,  92, 382,  72, 238, 232,
 240, 136, 347,  91,  64,  93,  94,  95,  96, 349,
  82, 233, 114, 115, 116, 117, 118, 119, 235, 120,
 314,  18,  16,  62,  72,  97, 432, 121,  73, 135,
  61, 431, 320, 143,  60, 236, 237, 234, 134, 134,
 226, 475, 420, 429, 246, 501, 147, 477, 429, 268,
 266, 137,  17, 425, 426, 427, 428, 430, 425, 426,
 427, 428, 430, 246, 246, 264, 146, 219, 220, 221,
 223, 222, 224, 225, 145, 227,  72, 456, 314, 246,
  73, 450, 337, 205, 437, 455, 313, 336, 149, 154,
 153, 158, 144, 350, 163, 164, 151, 157, 162, 335,
 159, 160, 161, 152, 156, 155, 134, 147, 324, 146,
 196, 181, 515, 323, 509, 195, 180, 145, 242,  72,
 508, 432, 413,  73, 167, 322, 431, 194, 179, 166,
 514, 149, 154, 153, 158, 165, 433, 163, 164, 151,
 157, 162, 511, 159, 160, 161, 152, 156, 155, 289,
 510, 292, 498, 295, 497, 298,  66, 399, 171, 297,
 294, 495, 494, 170,  66,  65,  72, 291,  72,  72,
  73, 262,  73,  73,  72, 169,  72, 507,  73, 243,
  73, 489, 288, 506, 480, 246, 493, 260, 469, 468,
 134,  72, 492, 467, 258,  73,  69,  70,  67,  68,
 401, 246, 334, 147,  69,  70,  67,  68, 246, 256,
 254, 252, 248, 245, 465, 451, 417, 416, 415, 414,
 345, 344, 464, 246, 246, 246, 246, 246, 463, 462,
 348, 311,  58, 412, 436, 461, 411, 410, 409, 408,
 407, 406,  59, 405, 404, 403, 402, 396, 319, 317,
 315, 318, 316, 343, 216,  59, 126, 352, 125, 353,
 354, 355, 124, 356, 123, 357, 435, 358,  77, 359,
  78, 360, 479, 361, 434, 362,  56, 363,  53, 342,
  21,  55,  51, 341,  52, 367,  21,  50,  40, 374,
  38,  36, 340, 339,  39, 338,  37,  35, 333, 332,
 331, 330, 390, 386, 392, 393, 329, 394, 328, 147,
 327, 397, 326, 325, 241, 228, 204, 203, 202, 201,
 200, 199, 198, 197, 193, 192, 191, 190, 189, 188,
 187, 186, 185, 184, 183, 182, 178, 177, 176, 175,
 174, 173, 172, 421, 168, 139, 122,  84,  21,  75,
 491,  76,  43,  21,  42, 500, 499, 496, 147, 474,
 473, 472, 471, 470, 134, 147, 147, 147, 147, 147,
 147, 147, 147, 147, 147, 147, 438, 466, 453, 373,
 312, 395, 310, 452, 454, 309, 383, 384, 212, 213,
 269, 299, 281, 272,  46,  47, 458, 459,  21, 457,
 418, 391, 389, 388, 387, 385, 380, 377, 376, 375,
 371, 370, 369, 368, 476, 366, 365, 478, 364, 321,
 308, 307, 306, 481, 305, 482, 304, 483, 303, 302,
 422, 301, 147, 147, 147, 147, 147, 439, 440, 441,
 442, 443, 444, 445, 446, 447, 448, 449, 490, 296,
 293, 290, 287, 286, 502, 285, 284, 503, 283, 280,
 504, 505, 279, 278, 277, 276, 275, 274, 512, 271,
 513, 209, 208, 207, 206, 138,  33,  30,  34,  31,
  23, 460,  24, 372,  83,  80,  63,  28,  26, 142,
 113,  99, 244,  90,  86,  81, 210,  57,  45,  20,
  44,   2, 112,  19, 484, 485, 486, 487, 488,  98,
  89, 351,  85, 230, 140,  29,  -1,  -1,  -1,  -1,
 379, 378,  41, 217,  27,  25,  22, 128,  48,  49,
 129, 127,  54,   9,   8,   7,   6,   5,   4,   3,
   1, 150, 148,  32,  74,   0,   0,   0,  79,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0, 130,   0,   0,   0,   0,   0,
   0, 141,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0, 215,   0,   0,   0,
   0,   0,   0,   0,   0, 229,   0,   0,   0,   0,
   0,   0,   0, 247, 249, 250, 251, 253, 255, 257,
 259, 261, 263, 265, 267,   0,   0,   0,   0,   0,
   0, 270,   0,   0, 273,   0,   0,   0,   0,   0,
   0,   0, 282,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0, 300,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0, 346 };
short yypact[]={

-245,-245,-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,
 178, 264, 272, 271, 261, 260,  75,  74,  72,-1000,
-1000,-1000, 133,-1000, 130, 176,-1000, 176,-1000,  66,
-1000,  62,  60,-1000,  16,-198,-1000,-202,-1000,-209,
-1000,-1000, 268, -54, 128,-1000, 129,  46, 128,-1000,
 267,-1000,-222,-1000,-1000, 266,-1000, 125,-1000,-326,
-271,-324,-268, 124,-1000,-1000,-1000,  38,  36,  32,
  30,-1000,-1000,-1000,-1000, 176,-231,-231, 256,-1000,
 123,-1000,-1000,  29,-152, -92,-1000,-1000, 122, -58,
-1000, 120, 119, 118, 117, 116, 115, 114,-105,-1000,
 113, 112, 111, 110, 109, 108, 107, 106, 105, 104,
 103, 102,-106,-1000, 101, 100,  99,  98,  97,  96,
  95,  94, -54, 255, 254, 253, 252, 170,-290, 170,
-1000,-1000,-1000,-1000,-1000,  28,-196,-1000,  93,-222,
-275,  92,-1000,-1000,-231,-109,-1000,-1000,-1000,  -3,
  -4,-186,-186,  -5,  -6,  -7, -22, -29, -45,-151,
-166,-167, 174,-1000,-1000,-1000,-326,-1000, 250, 173,
-271,-1000, 248, 247, 246, 245, 244, 243, 240, 172,
-324,-1000, 239, 237, 236, 234, 233, -37, 232, -52,
 231, -59, 230, -60, 171,-268,-1000, 212, 210, 209,
 207, 205, 203, 202, 201,-1000, 164, 161,   4, 159,
-143,-1000,  27,  26,  22,-201, 200,-108,-1000,  91,
  90,  88,  86,  84,  79,  78,  77,  76,-231,-1000,
-134,-1000,  73,-1000,  71,  70,  61,  57,  31,  -1,
  -2,-152,-243,   3,-135,-1000,-1000,-135,-1000,-135,
-135,-135,-1000,-135,-1000,-135,-1000,-135,-1000,-135,
-1000,-135,-1000,-135,-1000,-135,-1000,-135,-1000,-1000,
-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,
-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,
-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,
-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000, 199,
 197,-1000, 196,-1000, 170, 194, 193, 192, 191, 265,
-1000, 158,-1000,-196,-1000, 190, 189, 188,-1000,-1000,
 187,-276, 168, 186,-1000,-1000,-275,-1000, 185, 184,
 183, -54, 182, -54, -54,-204,-1000,-109,-1000,  21,
 -54, -62,  20,  19,  18,  17,  15,  14,  13,  12,
  11,  10,   7,-104,  -8,  -9, -10,-1000,-1000,-1000,
-1000,-1000, -11, 181,-1000,-1000,-1000,-1000,-188,-188,
-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,
-1000,-1000,-1000,-1000,-1000,-1000,-109,-235, -95,  50,
   9,-142,-231,-109,-109,-109,-109,-109,-109,-109,
-109,-109,-109,-109,-1000,-1000,-1000,-148, -12,-1000,
 -62,-1000, 157, -54,-144,-1000,-1000,-1000,-1000,-1000,
-1000, 180,-1000,-1000, 177,-1000, -54, 263,   8,   1,
 -13, 156, -34, -38, -39, 142, 141, 140, 139, 138,
-1000,-1000,-190, -54,-182,-1000, -54,  48,-1000,-1000,
 -43,-1000, -54,-1000, -54,-1000, -54,-1000,-1000,-1000,
-109,-109,-109,-109,-109,-1000, -46,-1000,-230, 131,
-1000, -35, -65, -66, 136, -73, -75, 135, 134,-1000,
-184,-1000,-1000, -54,-1000,-1000, -54,-1000,-1000, -54,
 -54,-1000, -44,-107, -77, -85,-1000, -54,-1000, -54,
-1000,-1000, -97,-115,-1000,-1000 };
short yypgo[]={

   0,   1,   0,  13, 529, 583,  73, 582, 535, 581,
 532,  49,   9, 580, 541, 579, 578, 577, 576, 575,
 574, 573, 539, 571, 536, 570, 567,  10, 566, 565,
 540, 564, 538, 563,   5, 561,   3, 560,   2, 559,
 558, 557, 556, 555, 554, 553,   6, 537, 552, 534,
 551, 550, 533, 549, 531, 542, 530 };
short yyr1[]={

   0,  13,  13,  14,  14,  14,  14,  14,  14,  14,
  14,  23,   1,  25,   1,  26,   1,  24,  24,  27,
  27,  27,  27,  27,  27,  15,  15,  28,  28,   2,
   2,   2,   2,   2,   2,   2,  29,  16,  31,  16,
  16,  16,  30,  30,  32,  32,  32,   3,   3,   3,
   3,   3,  33,  33,  33,  34,  34,  34,  35,  34,
  37,  34,  34,  34,  34,  34,  34,  34,  36,  38,
  38,  38,  38,  38,  38,  38,  38,  40,  39,  41,
  41,  41,  42,  17,  17,  43,  43,  43,  43,  44,
   8,  45,  45,  45,  46,  46,  46,  46,  46,  46,
  46,  46,  46,  18,  18,   5,   5,   5,   5,  47,
  47,  48,  48,  48,  49,  49,   4,   4,   6,   6,
   6,   6,   7,   7,   7,   7,   7,   7,   7,   7,
   7,   7,   7,   7,   7,   7,   7,   7,   7,   7,
   7,   7,   7,   7,   7,   7,   7,   7,   7,   7,
   7,   7,   9,   9,  50,  10,  10,  11,  11,  11,
  11,  12,  12,  12,  12,  12,  12,  19,  19,  51,
  51,  51,  52,  52,  52,  52,  52,  52,  52,  20,
  20,  53,  53,  53,  54,  54,  54,  54,  54,  54,
  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,
  21,  21,  55,  55,  55,  56,  56,  56,  56,  56,
  56,  56,  56,  22 };
short yyr2[]={

   0,   1,   2,   1,   1,   1,   1,   1,   1,   1,
   2,   0,   4,   0,   4,   0,   7,   1,   3,   3,
   3,   3,   3,   1,   1,   3,   2,   3,   5,   1,
   1,   6,   6,   4,   6,   1,   0,   4,   0,   4,
   2,   2,   1,   3,   3,   3,   5,   1,   1,   1,
   6,   3,   1,   3,   2,   3,   3,   3,   0,   4,
   0,   4,   3,   3,   3,   3,   3,   3,   3,   1,
   3,   3,   5,   3,   4,   2,   2,   0,   4,   1,
   3,   2,   5,   3,   2,   3,   5,   2,   2,   0,
   4,   1,   3,   2,   3,   1,   3,   3,   3,   3,
   3,   3,   3,   3,   2,   4,   6,   2,   2,   3,
   0,   1,   3,   2,   1,   3,   1,   4,   3,   1,
   1,   1,   8,   6,   8,  10,  12,   6,   8,   6,
   8,   6,   6,   6,  10,  12,   8,   8,  10,  10,
   2,   2,   2,   2,   2,   2,   2,   2,   2,   2,
   2,   2,   1,   1,   0,   4,   0,   5,   4,   7,
   0,   1,   1,   1,   1,   1,   1,   6,   2,   1,
   3,   2,   3,   3,   3,   3,   3,   3,   3,   6,
   2,   1,   3,   2,   3,   3,   3,   3,   3,   3,
   3,   3,   3,   3,   3,   3,   3,   3,   3,   3,
   6,   2,   1,   3,   2,   3,   3,   3,   3,   3,
   3,   3,   3,   1 };
short yychk[]={

-1000, -13, -14, -15, -16, -17, -18, -19, -20, -21,
 256, 274, 276, 277, 278, 275, 307, 337, 306, -14,
 -22, 260, -28, 256, 258, -29, 256, -31, 256, -43,
 256, 258,  -5, 256, 258, 262, 256, 262, 256, 262,
 256, -22, 261, 262, -30, -32, 258, 259, -30, -22,
 261, 256, 262, 256, -22, 261, 256, -47, 256, 266,
 272, 272, 272, 258,  -2, 259, 258, 300, 301, 298,
 299,  -1, 268, 272, -22, 261, 262, 262, 264, -22,
 258,  -8, 272, 258, 262, -48, -49, 334, 333, -51,
 -52, 314, 306, 316, 317, 318, 319, 336, -53, -54,
 338, 339, 340, 341, 342, 343, 344, 345, 346, 347,
 348, 349, -55, -56, 320, 321, 322, 323, 324, 325,
 327, 335, 262, 266, 266, 266, 266, -23, -26, -25,
 -32,  -3, 259, 258,  -1, 300, 272,  -3, 259, 262,
 -44, -47,  -4,  -6, 284, 266, 258,  -1,  -7, 280,
  -9, 288, 295, 282, 281, 297, 296, 289, 283, 292,
 293, 294, 290, 286, 287, 267, 261, 256, 262, 273,
 261, 256, 262, 262, 262, 262, 262, 262, 262, 273,
 261, 256, 262, 262, 262, 262, 262, 262, 262, 262,
 262, 262, 262, 262, 273, 261, 256, 262, 262, 262,
 262, 262, 262, 262, 262,  -2, 259, 259, 259, 259,
 -24, -27, 258, 259, 302, -24, 266, -33, -34, 303,
 304, 305, 307, 306, 308, 309, 276, 311, 262,  -8,
 -45, -46, 314, 326, 352, 333, 350, 351, 313, 279,
 315, 262,  -3,  -6, -10, 256, 270, -10, 256, -10,
 -10, -10, 256, -10, 256, -10, 256, -10, 256, -10,
 256, -10, 256, -10, 256, -10, 256, -10, 256, 256,
 -49, 259, 260, -52, 259, 259, 259, 259, 259, 259,
 259, 260, -54, 259, 259, 259, 259, 259, 259,  -1,
 259, 259,  -1, 259, 259,  -1, 259, 259,  -1, 260,
 -56, 259, 259, 259, 259, 259, 259, 259, 259, 261,
 261, 267, 261, 269, 261, 263, 265, 263, 265, 266,
 273, 259, 273, 261, 256, 262, 262, 262, 262, 262,
 262, 262, 262, 262,  -3, 273, 261, 256, 262, 262,
 262, 262, 262, 262, 262, 262,  -4, 285, 267, -11,
 268, -50, -11, -11, -11, -11, -11, -11, -11, -11,
 -11, -11, -11, -11, 259, 259, 259, -27, 259, 259,
 259, 259, 258, 261, -34, 259, 259, 259, -35, -37,
 259, 310, 312, 258, 259, 259, -46, 259, 259, 259,
  -2, 259,  -2,  -2,  -1,  -6, 266,  -2, -38, 259,
  -2, 302, 266, 266, 266, 266, 266, 266, 266, 266,
 266, 266, 266, 266, 267, 267, 267, 267, 259, -36,
 270, -36,  -6, 261, -12, 328, 329, 330, 331, 318,
 332, 261, 256, 271, 264, 256, 265, 266,  -3,  -6,
  -6,  -6,  -6,  -6,  -6,  -6,  -6,  -6,  -6,  -6,
 269, 267, -38, 261,  -2, 269, 261, 259, 259,  -2,
 258, 267, 261, 267, 261, 267, 261, 267, 267, 267,
 261, 261, 261, 261, 261, 271,  -2, 269,  -2, 264,
 267,  -2,  -2,  -2,  -6,  -6,  -6,  -6,  -6, 267,
 -12, 259, 267, 261, 267, 267, 261, 267, 267, 261,
 261, 269,  -2,  -2,  -2,  -2, 267, 261, 267, 261,
 267, 267,  -2,  -2, 267, 267 };
short yydef[]={

   0,  -2,   1,   3,   4,   5,   6,   7,   8,   9,
   0,   0,  36,  38,   0,   0,   0,   0,   0,   2,
  10, 213,   0,  26,   0,   0,  40,   0,  41,   0,
  84,   0,   0, 104, 110,   0, 168,   0, 180,   0,
 201,  25,   0,   0,   0,  42,   0,   0,   0,  83,
   0,  88,   0,  87, 103,   0, 108,   0, 107,   0,
   0,   0,   0,   0,  27,  29,  30,   0,   0,   0,
   0,  35,  -2,  13,  37,   0,   0,   0,   0,  39,
   0,  85,  89, 110,   0,   0, 111, 114,   0,   0,
 169,   0,   0,   0,   0,   0,   0,   0,   0, 181,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0, 202,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
  43,  44,  47,  48,  49,   0,  13,  45,   0,   0,
   0,   0, 105, 116,   0,   0, 119, 120, 121, 156,
 156, 156, 156, 156, 156, 156, 156, 156, 156, 156,
 156, 156,   0, 152, 153, 109,   0, 113,   0,   0,
   0, 171,   0,   0,   0,   0,   0,   0,   0,   0,
   0, 183,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0, 204,   0,   0,   0,
   0,   0,   0,   0,   0,  28,   0,   0,   0,   0,
   0,  17,  23,  24,   0,   0,   0,   0,  52,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,  86,
   0,  91,   0,  95,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0, 160, 140, 154, 160, 141, 160,
 160, 160, 142, 160, 143, 160, 144, 160, 145, 160,
 146, 160, 148, 160, 149, 160, 150, 160, 151, 147,
 112, 115, 167, 170, 172, 173, 174, 175, 176, 177,
 178, 179, 182, 184, 185, 186, 187, 188, 189, 190,
 191, 192, 193, 194, 195, 196, 197, 198, 199, 200,
 203, 205, 206, 207, 208, 209, 210, 211, 212,   0,
   0,  33,   0,  12,   0,   0,   0,   0,   0,   0,
  14,   0,  51,   0,  54,   0,   0,   0,  58,  60,
   0,   0,   0,   0,  46,  90,   0,  93,   0,   0,
   0,   0,   0,   0,   0,   0, 106,   0, 118,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,  18,  19,  21,
  20,  22,   0,   0,  53,  55,  56,  57,   0,   0,
  62,  63,  64,  65,  66,  67,  92,  94,  96,  97,
  98,  99, 100, 101, 102, 117,   0,   0,   0,  -2,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,  31,  32,  34,   0,   0,  59,
   0,  61,   0,   0,   0, 161, 162, 163, 164, 165,
 166,   0,  76, 155,   0,  75,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
  16,  50,   0,   0,   0, 158,   0,  70,  71,  73,
   0, 123,   0, 127,   0, 129,   0, 131, 132, 133,
   0,   0,   0,   0,   0,  68,   0, 157,   0,   0,
  74,   0,   0,   0,   0,   0,   0,   0,   0, 122,
   0,  72, 124,   0, 128, 130,   0, 136, 137,   0,
   0, 159,   0,   0,   0,   0, 125,   0, 134,   0,
 138, 139,   0,   0, 126, 135 };
#ifndef lint
static char yaccpar_sccsid[] = "@(#)yaccpar	4.1	(Berkeley)	2/11/83";
#endif

#
# define YYFLAG -1000
# define YYERROR goto yyerrlab
# define YYACCEPT return(0)
# define YYABORT return(1)

/*	parser for yacc output	*/

#ifdef YYDEBUG
int yydebug = 0; /* 1 for debugging */
#endif
YYSTYPE yyv[YYMAXDEPTH]; /* where the values are stored */
int yychar = -1; /* current input token number */
int yynerrs = 0;  /* number of errors */
short yyerrflag = 0;  /* error recovery flag */

yyparse() {

	short yys[YYMAXDEPTH];
	short yyj, yym;
	register YYSTYPE *yypvt;
	register short yystate, *yyps, yyn;
	register YYSTYPE *yypv;
	register short *yyxi;

	yystate = 0;
	yychar = -1;
	yynerrs = 0;
	yyerrflag = 0;
	yyps= &yys[-1];
	yypv= &yyv[-1];

 yystack:    /* put a state and value onto the stack */

#ifdef YYDEBUG
	if( yydebug  ) printf( "state %d, char 0%o\n", yystate, yychar );
#endif
		if( ++yyps> &yys[YYMAXDEPTH] ) { yyerror( "yacc stack overflow" ); return(1); }
		*yyps = yystate;
		++yypv;
		*yypv = yyval;

 yynewstate:

	yyn = yypact[yystate];

	if( yyn<= YYFLAG ) goto yydefault; /* simple state */

	if( yychar<0 ) if( (yychar=yylex())<0 ) yychar=0;
	if( (yyn += yychar)<0 || yyn >= YYLAST ) goto yydefault;

	if( yychk[ yyn=yyact[ yyn ] ] == yychar ){ /* valid shift */
		yychar = -1;
		yyval = yylval;
		yystate = yyn;
		if( yyerrflag > 0 ) --yyerrflag;
		goto yystack;
		}

 yydefault:
	/* default state action */

	if( (yyn=yydef[yystate]) == -2 ) {
		if( yychar<0 ) if( (yychar=yylex())<0 ) yychar = 0;
		/* look through exception table */

		for( yyxi=yyexca; (*yyxi!= (-1)) || (yyxi[1]!=yystate) ; yyxi += 2 ) ; /* VOID */

		while( *(yyxi+=2) >= 0 ){
			if( *yyxi == yychar ) break;
			}
		if( (yyn = yyxi[1]) < 0 ) return(0);   /* accept */
		}

	if( yyn == 0 ){ /* error */
		/* error ... attempt to resume parsing */

		switch( yyerrflag ){

		case 0:   /* brand new error */

		yyerrlab:
			++yynerrs;

		case 1:
		case 2: /* incompletely recovered error ... try again */

			yyerrflag = 3;

			/* find a state where "error" is a legal shift action */

			while ( yyps >= yys ) {
			   yyn = yypact[*yyps] + YYERRCODE;
			   if( yyn>= 0 && yyn < YYLAST && yychk[yyact[yyn]] == YYERRCODE ){
			      yystate = yyact[yyn];  /* simulate a shift of "error" */
			      goto yystack;
			      }
			   yyn = yypact[*yyps];

			   /* the current yyps has no shift onn "error", pop stack */

#ifdef YYDEBUG
			   if( yydebug ) printf( "error recovery pops state %d, uncovers %d\n", *yyps, yyps[-1] );
#endif
			   --yyps;
			   --yypv;
			   }

			/* there is no state on the stack with an error shift ... abort */

	yyabort:
			return(1);


		case 3:  /* no shift yet; clobber input char */

#ifdef YYDEBUG
			if( yydebug ) printf( "error recovery discards char %d\n", yychar );
#endif

			if( yychar == 0 ) goto yyabort; /* don't discard EOF, quit */
			yychar = -1;
			goto yynewstate;   /* try again in the same state */

			}

		}

	/* reduction by production yyn */

#ifdef YYDEBUG
		if( yydebug ) printf("reduce %d\n",yyn);
#endif
		yyps -= yyr2[yyn];
		yypvt = yypv;
		yypv -= yyr2[yyn];
		yyval = yypv[1];
		yym=yyn;
			/* consult goto table to find next state */
		yyn = yyr1[yyn];
		yyj = yypgo[yyn] + *yyps + 1;
		if( yyj>=YYLAST || yychk[ yystate = yyact[yyj] ] != -yyn ) yystate = yyact[yypgo[yyn]];
		switch(yym){
			
case 2:
# line 212 "parser.y"
{ yyerrok; stmtErr = FALSE; } break;
case 10:
# line 223 "parser.y"
{ yyerror("unknown statement type"); } break;
case 11:
# line 226 "parser.y"
{ currProbDist = new ProbDist(rand_dist); } break;
case 12:
# line 227 "parser.y"
{
                   if (!currProbDist->spread()) // if problem spreading prob
		     delete currProbDist;    // dispose of defective list
		   else 
		     yyval.probdist = currProbDist;      // push the list on semantic stack
		 } break;
case 13:
# line 234 "parser.y"
{ currProbDist = new ProbDist(determ_dist); } break;
case 14:
# line 235 "parser.y"
{
	           yyval.probdist = currProbDist;        // push the list on semantic stack
		 } break;
case 15:
# line 239 "parser.y"
{ currProbDist = new ProbDist(rand_dist); } break;
case 16:
# line 240 "parser.y"
{
		      currFile = (RandomFile*) FileSyms.lookup(yypvt[-2].string); 
		      if (currFile == NULL) { // no such symbol?
			 sprintf(errmsg, "undefined file:  %s", yypvt[-2].string);
		         yyerror(errmsg);
		      } else {
			 currProbDist->add(currFile->file);
		      }
		      if (!currProbDist->spread()) // if problem spreading prob
			  delete currProbDist;    // dispose of defective list
		      else 
			  yyval.probdist = currProbDist; // push the list on semantic stack
		 } break;
case 19:
# line 260 "parser.y"
{ currProbDist->add(yypvt[-2].string, yypvt[-0].floatval); } break;
case 20:
# line 263 "parser.y"
{ currProbDist->add(yypvt[-2].floatval, yypvt[-0].floatval); } break;
case 21:
# line 266 "parser.y"
{ currProbDist->add(yypvt[-2].string, (int) yypvt[-0].floatval); } break;
case 22:
# line 269 "parser.y"
{ currProbDist->add(yypvt[-2].floatval, (int) yypvt[-0].floatval); } break;
case 23:
# line 272 "parser.y"
{ currProbDist->add(yypvt[-0].string); } break;
case 24:
# line 275 "parser.y"
{ currProbDist->add(yypvt[-0].floatval); } break;
case 26:
# line 279 "parser.y"
{yyerror("bad real statement syntax");} break;
case 27:
# line 282 "parser.y"
{
                   if (!stmtErr)
		     if (!RealSyms.add(yypvt[-2].string, yypvt[-0].randreal))
		       delete yypvt[-0].randreal;             // some error adding symbol
		   stmtErr = FALSE;
		 } break;
case 28:
# line 289 "parser.y"
{
                   if (!stmtErr)
		     if (!RealSyms.add(yypvt[-2].string, yypvt[-0].randreal))
		       delete yypvt[-0].randreal;             // some error adding symbol
		   stmtErr = FALSE;
		 } break;
case 29:
# line 297 "parser.y"
{
                   currRealNum = new RandomReal;
		   currRealNum->dist = const_real;
		   currRealNum->val = yypvt[-0].floatval;
		   yyval.randreal = currRealNum;     // push RandomReal on semantic stack  
		 } break;
case 30:
# line 304 "parser.y"
{
                   currRealNum = (RandomReal*) RealSyms.lookup(yypvt[-0].string);
		   if (currRealNum == NULL) { // no such symbol?
		     sprintf(errmsg, "undefined real number:  %s", yypvt[-0].string);
		     yyerror(errmsg);
		   }
		   else
		     yyval.randreal = currRealNum;     // push RandomReal on semantic stk
		 } break;
case 31:
# line 314 "parser.y"
{
                   currRealNum = new RandomReal;
		   currRealNum->dist = uniform_real;
		   currRealNum->uniformLow = yypvt[-3].floatval;
		   currRealNum->uniformHigh = yypvt[-1].floatval;
		   yyval.randreal = currRealNum;     // push RandomReal on semantic stack  
		 } break;
case 32:
# line 322 "parser.y"
{
                   currRealNum = new RandomReal;
		   currRealNum->dist = rand_real;
		   currRealNum->uniformLow = yypvt[-3].floatval;
		   currRealNum->uniformHigh = yypvt[-1].floatval;
		   yyval.randreal = currRealNum;     // push RandomReal on semantic stack  
		 } break;
case 33:
# line 330 "parser.y"
{
                   currRealNum = new RandomReal;
		   currRealNum->dist = expntl_real;
		   currRealNum->mean = yypvt[-1].floatval;
		   yyval.randreal = currRealNum;     // push RandomReal on semantic stack  
		 } break;
case 34:
# line 337 "parser.y"
{
                   currRealNum = new RandomReal;
		   currRealNum->dist = nrm_real;
		   currRealNum->mean = yypvt[-3].floatval;
		   currRealNum->variance = yypvt[-1].floatval;
		   yyval.randreal = currRealNum;     // push RandomReal on semantic stack  
		 } break;
case 35:
# line 345 "parser.y"
{
	           currProbDist = yypvt[-0].probdist;    // grab distr from top of stack
	           if (currProbDist->allReals()) { // are all elements reals? 
	             currRealNum = new RandomReal;
		     currRealNum->dist = userDist_real;
		     currRealNum->userDist = currProbDist;
		     yyval.randreal = currRealNum;   // push RandomReal on semantic stack
		     }
		   else {                // at least one object not a real
		     yyerror("one or more undefined real nums in distribution");
		     delete currProbDist; // trash it
		   }
		 } break;
case 36:
# line 360 "parser.y"
{currFileInfo.type = data_file;} break;
case 38:
# line 361 "parser.y"
{currFileInfo.type = index_file;} break;
case 40:
# line 362 "parser.y"
{yyerror("bad file statement syntax");} break;
case 41:
# line 363 "parser.y"
{yyerror("bad index statement syntax");} break;
case 44:
# line 370 "parser.y"
{ 
                   CreateFile(yypvt[-2].string, yypvt[-0].randfile, 0); 
		   ResetFile();           // clear accumulated attributes
		 } break;
case 45:
# line 375 "parser.y"
{ 
	           currFile = yypvt[-0].randfile;
		   if (currFile->dist != const_file)
		     yyerror("only constant files may be unnamed");
		   else {
		     if (notafile((int) yypvt[-2].floatval))
		       CreateFile(NULL, yypvt[-0].randfile, (int) yypvt[-2].floatval); 
		     else
		       yyerror("cannot reuse an existing data/index file ID");
		   }
		   ResetFile();           // clear accumulated attributes
		 } break;
case 46:
# line 388 "parser.y"
{
		   if (yypvt[-0].randfile->dist != const_file)
		     yyerror("only constant files may be unnamed");
		   else {
		     if (yypvt[-2].floatval > yypvt[-4].floatval) 
		       for (i = (int) yypvt[-4].floatval; i <= (int) yypvt[-2].floatval; i++) {
			 if (stmtErr) break; // dont cascade errors
			 if (notafile(i)) {
			   currFile = new RandomFile; // replicate file
			   currFile->dist = const_file;
			   CreateFile(NULL, currFile, i);
			 }
			 else
			   yyerror("cannot reuse an existing data/index file ID");
		       }
		     else {
		       sprintf(errmsg, "%d..%d is an invalid range", (int) yypvt[-4].floatval, 
			       (int) yypvt[-2].floatval);
		       yyerror(errmsg);
		     }
		   }
		   ResetFile();           // clear accumulated attributes
		   (yypvt[-0].randfile)->fileNo = -1;     // mess up fileNo so destructor only
		                          // cleans up rand file and nothing
					  // else by mistake
		   delete yypvt[-0].randfile;             // only used as a model for clones
		 } break;
case 47:
# line 417 "parser.y"
{
		   if (notafile((int) yypvt[-0].floatval)) {
		     sprintf(errmsg, "undefined file:  %d", (int) yypvt[-0].floatval);
		     yyerror(errmsg);
		   }
		   else 
		     yyval.randfile = ConstFiles[(int) yypvt[-0].floatval]; // push file on semantic stack
		 } break;
case 48:
# line 426 "parser.y"
{
                   currFile = (RandomFile*) FileSyms.lookup(yypvt[-0].string); // find symbol
		   if (currFile == NULL){ // no such symbol?
		     sprintf(errmsg, "undefined file:  %s", yypvt[-0].string);
		     yyerror(errmsg);
		   }
		   else
		     yyval.randfile = currFile;       // push file on semantic stack
		 } break;
case 49:
# line 436 "parser.y"
{
	           currProbDist = yypvt[-0].probdist;    // grab distr from top of stack
	           if (currProbDist->allFiles(currFileInfo.type)) {
	             currFile = new RandomFile;
		     currFile->dist = userDist_file;
		     currFile->userDist = currProbDist;
		     yyval.randfile = currFile;       // push file on semantic stack
		   }
		   else {                 // at least one bad thing in dist
		     delete currProbDist; // trash it
		   }
		 } break;
case 50:
# line 449 "parser.y"
{

                   // Check existence of all files in distribution

		   for (i = (int) yypvt[-3].floatval; i <= (int) yypvt[-1].floatval; i++) {
		     if (notafile(i)) {
		       sprintf(errmsg, "range includes an undefined file:  %d",
			       i);
		       yyerror(errmsg);
		       break;
		     }
		     if (ConstFiles[i]->type != currFileInfo.type) {
		       yyerror("range includes both index and data files");
		       break;
		     }
		   }

		   if (i > (int) yypvt[-1].floatval) {        // all files checked out
		     currFile = new RandomFile;
		     currFile->dist = random_file;
		     currFile->randomLow = (int) yypvt[-3].floatval;
		     currFile->randomHigh = (int) yypvt[-1].floatval;
		     yyval.randfile = currFile;       // push file on semantic stack
		   }
		 } break;
case 51:
# line 475 "parser.y"
{
                   currFile = new RandomFile;
		   currFile->dist = const_file;
		   yyval.randfile = currFile;       // push file on semantic stack
		 } break;
case 54:
# line 485 "parser.y"
{yyerror("bad file/index attribute"); } break;
case 55:
# line 488 "parser.y"
{
		   if (notafile((int) yypvt[-0].floatval))
		     currFileInfo.fileNo = (int) yypvt[-0].floatval;
		   else
		     yyerror("cannot reuse an existing data/index file ID");
		 } break;
case 56:
# line 495 "parser.y"
{
		   if (yypvt[-0].floatval <= 0.0)
		     yyerror("tuplesize must be > 0");
		   else
		     currFileInfo.tupleSize = (int) yypvt[-0].floatval;
		 } break;
case 57:
# line 502 "parser.y"
{
		   if (yypvt[-0].floatval <= 0.0)
		     yyerror("number of tuples must be > 0");
		   else
		     currFileInfo.numTuples = (int) yypvt[-0].floatval;
		 } break;
case 58:
# line 509 "parser.y"
{ currIntSet = new IntegerSet(NumDiskNodes); } break;
case 59:
# line 510 "parser.y"
{ currFileInfo.nodeSet = currIntSet; } break;
case 60:
# line 512 "parser.y"
{currIntSet = new IntegerSet(DisksPerNode); } break;
case 61:
# line 513 "parser.y"
{ currFileInfo.diskSet = currIntSet; } break;
case 62:
# line 515 "parser.y"
{
	           currFileInfo.keySize = (int) yypvt[-0].floatval;
	         } break;
case 63:
# line 519 "parser.y"
{
	           currFileInfo.indexType = CLUSTERED;
	         } break;
case 64:
# line 523 "parser.y"
{
	           currFileInfo.indexType = UNCLUSTERED;
	         } break;
case 65:
# line 527 "parser.y"
{
                   currFile = (RandomFile*) FileSyms.lookup(yypvt[-0].string); // find symbol
		   
		   if (currFile == NULL){ // no such symbol?
		     sprintf(errmsg, "undefined file:  %s", yypvt[-0].string);
		     yyerror(errmsg);
		   }
		   else {
		     currFileInfo.indexedFile = currFile;
		     
		     if (currFile->dist != const_file)
		       yyerror("idices may only be created on constant files");

		     if (currFile->type != data_file)
		       yyerror("idices may only be created on data files");
		   }
	         } break;
case 66:
# line 545 "parser.y"
{
		   if (notafile((int) yypvt[-0].floatval)) {
		     sprintf(errmsg, "undefined file:  %d", (int) yypvt[-0].floatval);
		     yyerror(errmsg);
		   }
		   else {
		     currFileInfo.indexedFile = ConstFiles[(int) yypvt[-0].floatval];
		     
		     if (currFileInfo.indexedFile->dist != const_file)
		       yyerror("idices may only be created on constant files");

		     if (currFileInfo.indexedFile->type != data_file)
		       yyerror("idices may only be created on data files");
		   }
	         } break;
case 67:
# line 560 "parser.y"
{
	           currFileInfo.probHit = yypvt[-0].floatval;
		   if ((currFileInfo.probHit < 0) || 
		       (currFileInfo.probHit > 1.0)) {
		     yyerror("prob hit parameter doesn't make sense\n");
		   }
                 } break;
case 69:
# line 572 "parser.y"
{ currIntSet->add((int) yypvt[-0].floatval); } break;
case 70:
# line 574 "parser.y"
{ currIntSet->add((int) yypvt[-0].floatval); } break;
case 71:
# line 576 "parser.y"
{
	           if ((int) yypvt[-0].floatval >= (int) yypvt[-2].floatval)
		     for (i = (int) yypvt[-2].floatval; i <= (int) yypvt[-0].floatval; i++)
		       if (currIntSet->add(i) != 0) break;
		 } break;
case 72:
# line 582 "parser.y"
{ 
	           if ((int) yypvt[-0].floatval >= (int) yypvt[-2].floatval)
		     for (i = (int) yypvt[-2].floatval; i <= (int) yypvt[-0].floatval; i++)
		       if (currIntSet->add(i) != 0) break;
		 } break;
case 73:
# line 588 "parser.y"
{
	           currIntSet->setType = random_intset; // change from default
		   delete currIntSet->set; // kill fixed set storage
		   currIntSet->set = NULL;
		   currIntSet->randSet = new RandomSet(currIntSet->max);
	           currIntSet->randSet->elements = yypvt[-2].randreal;
	           currIntSet->randSet->cardinality = yypvt[-0].randreal;
		 } break;
case 74:
# line 597 "parser.y"
{
		      currFile = (RandomFile*) FileSyms.lookup(yypvt[-1].string); 
		      if (currFile == NULL) { // no such symbol?
			 sprintf(errmsg, "undefined file:  %s", yypvt[-1].string);
		         yyerror(errmsg);
		      } else {
			 currIntSet->add(currFile->file);
		      }
		 } break;
case 75:
# line 607 "parser.y"
{yyerror("bad set definition"); } break;
case 76:
# line 608 "parser.y"
{yyerror("bad set definition"); } break;
case 77:
# line 611 "parser.y"
{ currProbDist  = new ProbDist(determ_dist);
			  currProbDist2 = new ProbDist(determ_dist); } break;
case 81:
# line 618 "parser.y"
{yyerror("bad time sequence list");} break;
case 82:
# line 621 "parser.y"
{
                   if (yypvt[-3].floatval <= 0.0)
		     yyerror("population must be > 0");
		   else if (yypvt[-1].floatval <= 0.0)
		     yyerror("time must be > 0");
		   else {
		       currProbDist->add(yypvt[-3].floatval);
		       currProbDist2->add(yypvt[-1].floatval);
		   }
	         } break;
case 84:
# line 634 "parser.y"
{yyerror("bad jobclass statement syntax");} break;
case 85:
# line 638 "parser.y"
{ CreateJobClass(yypvt[-2].string, yypvt[-0].jclass); } break;
case 86:
# line 641 "parser.y"
{ CreateJobClass(yypvt[-2].string, yypvt[-0].jclass); } break;
case 87:
# line 643 "parser.y"
{yyerror("bad job class statement"); } break;
case 88:
# line 644 "parser.y"
{yyerror("bad job class statement"); } break;
case 89:
# line 647 "parser.y"
{currJClass = new JobClass;} break;
case 90:
# line 648 "parser.y"
{
		   yyval.jclass = currJClass;      // push job class on semantic stack
		 } break;
case 93:
# line 656 "parser.y"
{yyerror("bad job class definition list");} break;
case 94:
# line 659 "parser.y"
{
		   if (yypvt[-0].floatval < 0.0)
		     yyerror("population must be >= 0");
		   else if (yypvt[-0].floatval > MAX_TERMINALS) {
		     sprintf(errmsg, "population greater than MAX_TERMINALS");
		     yyerror(errmsg);
		   } else
		     currJClass->population = (int) yypvt[-0].floatval;
		 } break;
case 95:
# line 669 "parser.y"
{

	           /* note we cannot set stopCount to the population at this
		      point (which is what it is supposed to be), since 
		      the POPULATION= keyword may not have been recognized
		      yet.  Instead, we just change the value to something
		      non-negative, and the JobClass::start method will 
		      set its value to equal the job class population before
		      kicking off the terminals of this class. */

                   currJClass->stopCount = 0;
		 } break;
case 96:
# line 682 "parser.y"
{
		   if (yypvt[-0].floatval <= 0)
		     yyerror("completions must be > 0");
		   else
		     currJClass->maxCompletions = (int) yypvt[-0].floatval;
	         } break;
case 97:
# line 689 "parser.y"
{
	           currJClass->maxMPL = (int) yypvt[-0].floatval;
		 } break;
case 98:
# line 694 "parser.y"
{
	           currJClass->waitSecs = yypvt[-0].randreal;
		 } break;
case 99:
# line 698 "parser.y"
{
	           currJClass->statWaitSecs = yypvt[-0].floatval;
		 } break;
case 100:
# line 702 "parser.y"
{
	           currJClass->thinksecs = yypvt[-0].randreal;
		 } break;
case 101:
# line 706 "parser.y"
{
		   currJClass->interArrivalSecs = yypvt[-0].randreal;
		   OpenClasses = TRUE; // let sim.c know open classes exist
		 } break;
case 102:
# line 711 "parser.y"
{
		   currProbDist = yypvt[-0].probdist;
		   if (currProbDist->allPlans()) { // are all objects plans?
		     currJClass->plans = currProbDist;       
		   } else {
		     yyerror("one or more undefined plans in distribution");
		     delete currProbDist;         // trash it
		   }
		 } break;
case 104:
# line 723 "parser.y"
{yyerror("bad plan statement syntax");} break;
case 105:
# line 727 "parser.y"
{ CreatePlan(yypvt[-3].string, yypvt[-0].randplan);
		     ResetPlan();
		   } break;
case 106:
# line 731 "parser.y"
{ CreatePlan(yypvt[-3].string, yypvt[-0].randplan);
		     ResetPlan();
		   } break;
case 107:
# line 735 "parser.y"
{yyerror("bad plan statement"); } break;
case 108:
# line 736 "parser.y"
{yyerror("bad plan statement"); } break;
case 113:
# line 750 "parser.y"
{yyerror("bad plan attribute"); } break;
case 114:
# line 753 "parser.y"
{
                     currPlanInfo.stats = TRUE;
		 } break;
case 115:
# line 757 "parser.y"
{
	             currPlanInfo.maxmpl = (int) yypvt[-0].floatval;
		 } break;
case 116:
# line 762 "parser.y"
{
                   yyval.randplan = yypvt[-0].randplan;  // just pass it on up
		 } break;
case 117:
# line 766 "parser.y"
{
	           yyerror("statement not supported yet");
		 } break;
case 118:
# line 771 "parser.y"
{
                   yyval.randplan = yypvt[-1].randplan;             // just pass it on up
		 } break;
case 119:
# line 775 "parser.y"
{
		   if ((yyval.randplan = (RandomPlan*) PlanSyms.lookup(yypvt[-0].string)) == NULL) {
		     sprintf(errmsg, "undefined plan:  %s", yypvt[-0].string);
		     yyerror(errmsg);
		   }
		 } break;
case 120:
# line 782 "parser.y"
{
                   currProbDist = yypvt[-0].probdist;
	           if (currProbDist->allPlans()) { // are all objects plans?
		     currPlan = new RandomPlan;
		     currPlan->dist = userDist_plan;
		     currPlan->userDist = currProbDist;
		     yyval.randplan = currPlan;
		   }
		   else {                 
		     yyerror("one or more undefined plans in distribution");
		     delete currProbDist;     // trash it
		     }
		 } break;
case 121:
# line 796 "parser.y"
{ 
	           yyval.randplan = yypvt[-0].randplan;  // just pass it on up
		 } break;
case 122:
# line 801 "parser.y"
{
                   // proj(Query, retain %) 
	           
		   yyerror("statement not supported yet");
                 } break;
case 123:
# line 807 "parser.y"
{
                   //  no selection predicate, no projection 
		   
		   if (yypvt[-5].stype == file_scan && yypvt[-1].randfile->type != data_file)
		     yyerror("cannot perform a file scan on an index file");
		   if (yypvt[-5].stype == index_scan && yypvt[-1].randfile->type != index_file)
		     yyerror("cannot perform an index scan on a data file");

		   currPlan = new RandomPlan;
		   currPlan->dist = const_plan;
		   currPlan->opCode = SELECT;
		   currPlan->file = yypvt[-1].randfile;
		   currPlan->sel = new RandomReal;
		   currPlan->sel->dist = const_real;
		   currPlan->sel->val = 1.0;
		   if (yypvt[-5].stype == index_scan) { //need to default index selectivity?
		     currPlan->isel = new RandomReal;
		     currPlan->isel->dist = const_real;
		     currPlan->isel->val = 1.0;
		   }
		   currPlan->proj = new RandomReal;
		   currPlan->proj->dist = const_real;
		   currPlan->proj->val = 1.0;
		   if (yypvt[-4].iset != NULL) {       // if site assmt expr present
		     currPlan->sites = (yypvt[-4].iset)->set;
		     currPlan->sizeSites = (yypvt[-4].iset)->card;
		     currPlan->siteSet = (yypvt[-4].iset)->randSet;
		   }
		   currPlan->memReq = yypvt[-3].mreq;
		   yyval.randplan = currPlan;
                 } break;
case 124:
# line 839 "parser.y"
{
                   // fscan(File, selectivity %) 
		   
		   if (yypvt[-7].stype == file_scan && yypvt[-3].randfile->type != data_file)
		     yyerror("cannot perform a file scan on an index file");
		   if (yypvt[-7].stype == index_scan && yypvt[-3].randfile->type != index_file)
		     yyerror("cannot perform an index scan on a data file");
		   if (yypvt[-7].stype == index_scan)
		     yyerror("index scan requires 2 selectivities:  operator sel and index sel");

		   currPlan = new RandomPlan;
		   currPlan->dist = const_plan;
		   currPlan->opCode = SELECT;   
		   currPlan->file = yypvt[-3].randfile;
		   currPlan->sel = yypvt[-1].randreal;
		   currPlan->proj = new RandomReal;
		   currPlan->proj->dist = const_real;
		   currPlan->proj->val = 1.0;
		   if (yypvt[-6].iset != NULL) {       // if site assmt expr present
		     currPlan->sites = (yypvt[-6].iset)->set;
		     currPlan->sizeSites = (yypvt[-6].iset)->card;
		     currPlan->siteSet = (yypvt[-6].iset)->randSet;
		   }
		   currPlan->memReq = yypvt[-5].mreq;
		   yyval.randplan = currPlan;
                 } break;
case 125:
# line 867 "parser.y"
{
                   // fscan(File, selectivity %, project retain %) -or-
                   // iscan(File, op selectivity %, index selectivity %)
		   
		   if (yypvt[-9].stype == file_scan && yypvt[-5].randfile->type != data_file)
		     yyerror("cannot perform a file scan on an index file");
		   if (yypvt[-9].stype == index_scan && yypvt[-5].randfile->type != index_file)
		     yyerror("cannot perform an index scan on a data file");

		   currPlan = new RandomPlan;
		   currPlan->dist = const_plan;
		   currPlan->opCode = SELECT;   
		   currPlan->file = yypvt[-5].randfile;
		   currPlan->sel = yypvt[-3].randreal;
		   if (yypvt[-9].stype == file_scan)
		     currPlan->proj = yypvt[-1].randreal;
		   else {
		     currPlan->isel = yypvt[-1].randreal;  // must default projection to 100%
		     currPlan->proj = new RandomReal;
		     currPlan->proj->dist = const_real;
		     currPlan->proj->val = 1.0;
		   }
		   if (yypvt[-8].iset != NULL) {       // if site assmt expr present
		     currPlan->sites = (yypvt[-8].iset)->set;
		     currPlan->sizeSites = (yypvt[-8].iset)->card;
		     currPlan->siteSet = (yypvt[-8].iset)->randSet;
		   }
		   currPlan->memReq = yypvt[-7].mreq;
		   yyval.randplan = currPlan;
                 } break;
case 126:
# line 899 "parser.y"
{
                   // iscan(File, op sel %, index sel %, proj %)
		   
		   if (yypvt[-11].stype == file_scan)
		     yyerror("illegal file scan syntax");
		   if (yypvt[-11].stype == index_scan && yypvt[-7].randfile->type != index_file)
		     yyerror("cannot perform an index scan on a data file");

		   currPlan = new RandomPlan;
		   currPlan->dist = const_plan;
		   currPlan->opCode = SELECT;   
		   currPlan->file = yypvt[-7].randfile;
		   currPlan->sel = yypvt[-5].randreal;
		   currPlan->isel = yypvt[-3].randreal;
		   currPlan->proj = yypvt[-1].randreal;
		   if (yypvt[-10].iset != NULL) {       // if site assmt expr present
		     currPlan->sites = (yypvt[-10].iset)->set;
		     currPlan->sizeSites = (yypvt[-10].iset)->card;
		     currPlan->siteSet = (yypvt[-10].iset)->randSet;
		   }
		   currPlan->memReq = yypvt[-9].mreq;
		   yyval.randplan = currPlan;
                 } break;
case 127:
# line 923 "parser.y"
{
	           // store(query) - write query result in new file on disk 0

		   currPlan = new RandomPlan;
		   currPlan->dist = const_plan;
		   currPlan->opCode = STORE;
		   currPlan->lchild = yypvt[-1].randplan;  // unary ops use left child
		   currPlan->disk = new RandomReal; // default is random disk
		   currPlan->disk->dist = rand_real; 
		   currPlan->disk->uniformLow = 0;
		   currPlan->disk->uniformHigh = DisksPerNode-1;
		   if (yypvt[-4].iset != NULL) {       // if site assmt expr present
		     currPlan->sites = (yypvt[-4].iset)->set;
		     currPlan->sizeSites = (yypvt[-4].iset)->card;
		     currPlan->siteSet = (yypvt[-4].iset)->randSet;
		   }
		   currPlan->memReq = yypvt[-3].mreq;
		   yyval.randplan = currPlan;
		 } break;
case 128:
# line 943 "parser.y"
{
	           // store(query, disk #) - write query result in new file

		   currPlan = new RandomPlan;
		   currPlan->dist = const_plan;
		   currPlan->opCode = STORE;
		   currPlan->lchild = yypvt[-3].randplan;  // unary ops use left child
		   currPlan->disk = yypvt[-1].randreal;    
		   if (yypvt[-6].iset != NULL) {       // if site assmt expr present
		     currPlan->sites = (yypvt[-6].iset)->set;
		     currPlan->sizeSites = (yypvt[-6].iset)->card;
		     currPlan->siteSet = (yypvt[-6].iset)->randSet;
		   }
		   currPlan->memReq = yypvt[-5].mreq;
		   yyval.randplan = currPlan;
		 } break;
case 129:
# line 960 "parser.y"
{
	           // termsend(query) - send query result to terminal

		   currPlan = new RandomPlan;
		   currPlan->dist = const_plan;
		   currPlan->opCode = TERMOP;
		   currPlan->lchild = yypvt[-1].randplan;  // unary ops use left child
		   if (yypvt[-4].iset != NULL) {       // if site assmt expr present
		     currPlan->sites = (yypvt[-4].iset)->set;
		     currPlan->sizeSites = (yypvt[-4].iset)->card;
		     currPlan->siteSet = (yypvt[-4].iset)->randSet;
		   }
		   if (yypvt[-3].mreq != NULL) delete yypvt[-3].mreq;  // memory specs are ignored
		   yyval.randplan = currPlan;
		 } break;
case 130:
# line 976 "parser.y"
{
                   // aggr(Query, cost/tuple) 
	           
		   yyerror("statement not supported yet");
	         } break;
case 131:
# line 982 "parser.y"
{
                   // sort(query) 
	           
		   yyerror("statement not supported yet");
	         } break;
case 132:
# line 988 "parser.y"
{
                   // sortunique(query) 
	           
		   yyerror("statement not supported yet");
	         } break;
case 133:
# line 994 "parser.y"
{
                   // unique(query) 
	           
		   yyerror("statement not supported yet");
	         } break;
case 134:
# line 1001 "parser.y"
{
                   // hjoin(query, query, join probability) 
		   
		   currPlan = new RandomPlan;
		   currPlan->dist = const_plan;
		   currPlan->opCode = JOIN;   // more specific please!!!
		   currPlan->lchild = yypvt[-5].randplan;
		   currPlan->rchild = yypvt[-3].randplan;
		   currPlan->sel = yypvt[-1].randreal;
		   currPlan->proj = new RandomReal; // default proj% to 100%
		   currPlan->proj->dist = const_real;
		   currPlan->proj->val = 1.0;
		   if (yypvt[-8].iset != NULL) {       // if site assmt expr present
		     currPlan->sites = (yypvt[-8].iset)->set;
		     currPlan->sizeSites = (yypvt[-8].iset)->card;
		     currPlan->siteSet = (yypvt[-8].iset)->randSet;
		   }
		   currPlan->memReq = yypvt[-7].mreq;
		   yyval.randplan = currPlan;
	         } break;
case 135:
# line 1023 "parser.y"
{
                   // hjoin(query, query, join probability, project retain %) 
		   
		   currPlan = new RandomPlan;
		   currPlan->dist = const_plan;
		   currPlan->opCode = JOIN;   // more specific please!!!
		   currPlan->lchild = yypvt[-7].randplan;
		   currPlan->rchild = yypvt[-5].randplan;
		   currPlan->sel = yypvt[-3].randreal;
		   currPlan->proj = yypvt[-1].randreal;
		   if (yypvt[-10].iset != NULL) {       // if site assmt expr present
		     currPlan->sites = (yypvt[-10].iset)->set;
		     currPlan->sizeSites = (yypvt[-10].iset)->card;
		     currPlan->siteSet = (yypvt[-10].iset)->randSet;
		   }
		   currPlan->memReq = yypvt[-9].mreq;
		   yyval.randplan = currPlan;
	         } break;
case 136:
# line 1042 "parser.y"
{
                   // xprod(query, query) 
	           
		   yyerror("statement not supported yet");
	         } break;
case 137:
# line 1048 "parser.y"
{
                   // union(query, query) 
	           
		   yyerror("statement not supported yet");
	         } break;
case 138:
# line 1055 "parser.y"
{
                   // intersect(query, query, % of union) 
	           
		   yyerror("statement not supported yet");
	         } break;
case 139:
# line 1062 "parser.y"
{
                   // difference(query, query, % of first file retained) 
	           
		   yyerror("statement not supported yet");
	         } break;
case 140:
# line 1068 "parser.y"
{yyerror("bad project syntax");} break;
case 141:
# line 1069 "parser.y"
{yyerror("bad scan syntax");} break;
case 142:
# line 1070 "parser.y"
{yyerror("bad aggregation syntax");} break;
case 143:
# line 1071 "parser.y"
{yyerror("bad sort syntax");} break;
case 144:
# line 1072 "parser.y"
{yyerror("bad sortunique syntax");} break;
case 145:
# line 1073 "parser.y"
{yyerror("bad unique syntax");} break;
case 146:
# line 1074 "parser.y"
{yyerror("bad hash join syntax");} break;
case 147:
# line 1076 "parser.y"
{yyerror("bad index replicate join syntax");} break;
case 148:
# line 1077 "parser.y"
{yyerror("bad xprod syntax");} break;
case 149:
# line 1078 "parser.y"
{yyerror("bad union syntax");} break;
case 150:
# line 1079 "parser.y"
{yyerror("bad intersection syntax");} break;
case 151:
# line 1080 "parser.y"
{yyerror("bad difference syntax");} break;
case 152:
# line 1083 "parser.y"
{ yyval.stype = index_scan; } break;
case 153:
# line 1084 "parser.y"
{ yyval.stype = file_scan; } break;
case 154:
# line 1087 "parser.y"
{ currIntSet = new IntegerSet(NumDiskNodes); } break;
case 155:
# line 1088 "parser.y"
{

		   // Verifies the numbers are reasonable for node IDs
		   // Should really check for duplicate node IDs too !!!!

		   if (currIntSet->setType == constant_intset) {

		     for (i = 0; i < currIntSet->card; i++) 
		       if (notanode(currIntSet->set[i])) break;

		     if (i < currIntSet->card) { // some node didnt check out
		       yyerror("one or more invalid nodes specified");
		       delete currIntSet;
		       yyval.iset = NULL;           // tell op to take defaults
		     } else {
		       yyval.iset = currIntSet;     // give op the set of sites
		     }
		   } else {
		     yyval.iset = currIntSet;       // cant verify random sets
		   }
		 } break;
case 156:
# line 1110 "parser.y"
{yyval.iset = NULL;} break;
case 157:
# line 1113 "parser.y"
{
                   yyerror("obsolete memory syntax");
		 } break;
case 158:
# line 1117 "parser.y"
{
                   currMemReq = new MemReq;
		   currMemReq->min = currMemReq->max = yypvt[-2].randreal;
		   currMemReq->minUnits = currMemReq->maxUnits = yypvt[-1].intval;
		   yyval.mreq = currMemReq;
		 } break;
case 159:
# line 1124 "parser.y"
{
                   currMemReq = new MemReq;
		   currMemReq->min = yypvt[-5].randreal;
		   currMemReq->minUnits = yypvt[-4].intval;
		   currMemReq->max = yypvt[-2].randreal;
		   currMemReq->maxUnits = yypvt[-1].intval;
		   yyval.mreq = currMemReq;
		 } break;
case 160:
# line 1133 "parser.y"
{yyval.mreq = NULL;} break;
case 161:
# line 1136 "parser.y"
{yyval.intval = BUFPOOL_UNITS;} break;
case 162:
# line 1137 "parser.y"
{yyval.intval = MINREQ_UNITS;} break;
case 163:
# line 1138 "parser.y"
{yyval.intval = MAXREQ_UNITS;} break;
case 164:
# line 1139 "parser.y"
{yyval.intval = BYTE_UNITS;} break;
case 165:
# line 1140 "parser.y"
{yyval.intval = KBYTE_UNITS;} break;
case 166:
# line 1141 "parser.y"
{yyval.intval = MBYTE_UNITS;} break;
case 168:
# line 1144 "parser.y"
{yyerror("bad nodes statement syntax");} break;
case 171:
# line 1150 "parser.y"
{yyerror("bad node definition syntax"); } break;
case 172:
# line 1153 "parser.y"
{
                   if ((int) yypvt[-0].floatval > MAX_NODES-1 || (int) yypvt[-0].floatval <= 0) {
     		     sprintf(errmsg, "number of disk nodes must be between 0 and %d", MAX_NODES);
		     yyerror(errmsg);
		   }
		   else {
                     NumDiskNodes = (int) yypvt[-0].floatval; 
		     NumNodes = NumDiskNodes+1;
		   }
		 } break;
case 173:
# line 1164 "parser.y"
{ 
	           if ((int) yypvt[-0].floatval > MAX_DISKSPERNODE || (int) yypvt[-0].floatval <= 0) {
		     sprintf(errmsg, "number of disks per node must be between 0 and  %d", MAX_DISKSPERNODE);
		     yyerror(errmsg);
		   }
		   else 
		     DisksPerNode = (int) yypvt[-0].floatval; 
		 } break;
case 174:
# line 1173 "parser.y"
{ 
	           if (yypvt[-0].floatval <= 0.0)
		     yyerror("number of cpus per node must be > zero");
		   else
		     CPUsPerNode = (int) yypvt[-0].floatval;
		 } break;
case 175:
# line 1180 "parser.y"
{
	           if (yypvt[-0].floatval <= 0.0)
		     yyerror("MIPS must be > zero");
		   else
		     Mips = yypvt[-0].floatval;
		 } break;
case 176:
# line 1187 "parser.y"
{ 
	           if (yypvt[-0].floatval <= 0.0)
		     yyerror("node memory must be > zero");
		   else
		     KBytesPerNode = (int) yypvt[-0].floatval;
		 } break;
case 177:
# line 1194 "parser.y"
{ 
	           if (yypvt[-0].floatval <= 0.0)
		     yyerror("page size must be > zero");
		   else
		     PageSize = (int) yypvt[-0].floatval;
		 } break;
case 178:
# line 1201 "parser.y"
{ Inst_StartIO = (int) yypvt[-0].floatval; } break;
case 180:
# line 1206 "parser.y"
{yyerror("bad network statement syntax");} break;
case 183:
# line 1213 "parser.y"
{yyerror("bad network definition syntax"); } break;
case 184:
# line 1216 "parser.y"
{
		   printf("maxpacketbytes not yet supported.  Ignored\n");
		 } break;
case 185:
# line 1220 "parser.y"
{
	           if (yypvt[-0].floatval <= 0.0)
		     yyerror("max requests must be > zero");
		   else
		     MaxRequests = (int) yypvt[-0].floatval;
		 } break;
case 186:
# line 1228 "parser.y"
{ GlobKBytes = (int) yypvt[-0].floatval; } break;
case 187:
# line 1231 "parser.y"
{LocKBytes = (int) yypvt[-0].floatval; } break;
case 188:
# line 1234 "parser.y"
{ MinSendProtocol = (int) yypvt[-0].floatval; } break;
case 189:
# line 1237 "parser.y"
{ SendProtocolPerByte = (int) yypvt[-0].floatval; } break;
case 190:
# line 1240 "parser.y"
{ yyerror("piecewise linear costs not yet supported.\n"); } break;
case 191:
# line 1243 "parser.y"
{ MinRecvProtocol = (int) yypvt[-0].floatval; } break;
case 192:
# line 1246 "parser.y"
{ RecvProtocolPerByte = (int) yypvt[-0].floatval; } break;
case 193:
# line 1249 "parser.y"
{ yyerror("piecewise linear costs not yet supported.\n"); } break;
case 194:
# line 1252 "parser.y"
{MinDelay = yypvt[-0].floatval; } break;
case 195:
# line 1255 "parser.y"
{DelayPerByte = yypvt[-0].floatval; } break;
case 196:
# line 1258 "parser.y"
{ yyerror("piecewise linear costs not yet supported.\n"); } break;
case 197:
# line 1261 "parser.y"
{ MinShortInstr = (int) yypvt[-0].floatval; } break;
case 198:
# line 1264 "parser.y"
{ ShortInstrPerByte = (int) yypvt[-0].floatval; } break;
case 199:
# line 1267 "parser.y"
{ yyerror("piecewise linear costs not yet supported.\n"); } break;
case 201:
# line 1272 "parser.y"
{yyerror("bad disks statement syntax");} break;
case 204:
# line 1278 "parser.y"
{yyerror("bad disk definition syntax"); } break;
case 205:
# line 1281 "parser.y"
{
                   if (yypvt[-0].floatval <= 0.0)
		     yyerror("pages per cylinder must be > 0");
		   else
		     PagesPerCyl = (int) yypvt[-0].floatval;
		 } break;
case 206:
# line 1288 "parser.y"
{ 
                   if (yypvt[-0].floatval <= 0.0)
		     yyerror("seek factor must be > 0");
		   else
		     SeekFactor = yypvt[-0].floatval; 
		 } break;
case 207:
# line 1295 "parser.y"
{ MinLatency = yypvt[-0].floatval; } break;
case 208:
# line 1297 "parser.y"
{ 
                   if (yypvt[-0].floatval <= 0.0)
		     yyerror("rotation speed must be > 0");
		   else
		     DiskRotation = yypvt[-0].floatval; 
		 } break;
case 209:
# line 1304 "parser.y"
{ SettleTime = yypvt[-0].floatval; } break;
case 210:
# line 1306 "parser.y"
{ XferRate = yypvt[-0].floatval; } break;
case 211:
# line 1308 "parser.y"
{ DiskCachePages = (int) yypvt[-0].floatval; } break;
case 212:
# line 1310 "parser.y"
{ DiskCacheContexts = (int) yypvt[-0].floatval; } break;
case 213:
# line 1313 "parser.y"
{ yyerrok; stmtErr = FALSE; } break; 
		}
		goto yystack;  /* stack new state and value */

	}
