///////////////////////////////////////////////////////////////////////////////
//
//  Name:        mailbox.h
//
//  Title:       UW database simulator mailbox class definitions.
//
//               The mbox class is an enhancement to the CSIM mailbox class 
//               which allows us to charge CSIM CPU facilities for message 
//               receive protocol pathlength costs.  It is a generic class and
//               is not intended for external use outside of its subclasses.
//
//               The MailBox and TermMailBox classes are futher refinements
//               which just indicates whether the owner is a terminal or a
//               node (send/receive processing is different for each case).
//
//  Change Activity:
//
//     08/25/91 (KPB) - Initial version.
//
///////////////////////////////////////////////////////////////////////////////

#define NETPKTSIZE 8192

class Msg;
class Node;
class Op;
class Terminal;

class mbox: public mailbox {        // enhanced CSIM mailbox
  public:
    int type;                       // TERMINAL or NODE
   
    mbox(char* name): mailbox(name) {};
    Msg* recv();                    // special version of receive
};

///////////////////////////////////////////////////////////////////////////////

class Mailbox: public mbox {
  private:
    static int numMbox;
    static Mailbox* freeMailbox;
  public:
    Node* owningNode;
    Op* owningOp;
    storage *msgPool;
    Mailbox* nextFreeBox; // free list chain for efficient stg mgt
    
    Mailbox(char* name, Node* owningNode, Op* owningOp = NULL);
    ~Mailbox() { if (msgPool != NULL) delete msgPool; }

    Msg* recv();
    inline void* operator new(size_t size);       // new is overloaded
    inline void  operator delete(void* oldBox);   // delete is overloaded
};

///////////////////////////////////////////////////////////////////////////////

class TermMailbox: public mbox {    // enhanced mbox
  public:
    Terminal* owningTerm;
    
    TermMailbox(char* name, Terminal* owningTerm);
};

//////////////////////////////////////////////////////////////////////////////

inline void* Mailbox::operator new(size_t size) {
    Mailbox *newBox;

    if (freeMailbox == NULL) {
	newBox = (Mailbox *) new char[size];
	if (Flags[TRACE_LEAK])
	  printf("new Mbox %d\n", ++numMbox);
    } else {
        newBox = freeMailbox;
	freeMailbox = freeMailbox->nextFreeBox;
    }
    return newBox;
};

//////////////////////////////////////////////////////////////////////////////

inline void Mailbox::operator delete(void* oldBox) {
    ((Mailbox *) oldBox)->nextFreeBox = freeMailbox;
    freeMailbox = (Mailbox *) oldBox;
};
