#include <string.h>
#include "dbsim.h"
#include "object.h"
#include "node.h"
#include "msg.h"
#include "mailbox.h"
#include "query.h"
#include "sched.h"
#include "language.h"
#include "bufmgr.h"

///////////////////////////////////////////////////////////////////////////////
//
//    Name:                GSched.c
//
//    Function:        Global Scheduler process
//                                 
//    Change Activity:
//
//         08/xx/91 (KPB) - Initial version.
//
///////////////////////////////////////////////////////////////////////////////




// Global variables to maintain the average MPL of the system
double PrevMPLChange = 0;  // last time a query admitted/departed
int CurrentMPL = 0;


extern Mailbox* GlobalSchedPort;
extern MemoryQueue *GlobalMemoryWaitQueue;
extern QueryQueue  *GlobalMPLWaitQueue;
extern initialSchedule(QueryMsg*, Schedule*&, Schedule*&);
extern allocateResources(QueryMsg*, Schedule*&, Schedule*&);
extern void QueryScheduler(Node* ,QueryMsg*, Schedule*, Schedule*);
extern CanRun(RootPlan*);
extern void DeleteSched(Schedule*&);
extern bool ResetBufferFlag;
extern int NumNodes;
extern Node* Nodes[];

void startQuery(Node*, QueryMsg*);
void globalScheduler(Node*); 


void globalScheduler(Node* gSchedNode) 
{
    JobClass *queryClass;
    QueryMsg *reqMsg;                                             
    int		transCounter=0, nodeNum, e;

    create("GlobalScheduler");  
    if (Flags[TRACE_CONTROL]) 
      printf("Global scheduler process started on node %d\n", gSchedNode->id);


    while (ActiveTerms > 0) 
    {    
      // while there are people to listen to
        // Waits for requests from terminals 

        reqMsg = (QueryMsg*) GlobalSchedPort->recv();

        // Stamps arriving query with arrival time for response time 
	// computation.

	((RootPlan*) reqMsg->query)->arrivalTime = clock;

	// WARNING: reset buffer flag works only if there is just one terminal
	//   submitting a single query to the global scheduler. it will not
	//   work if used in other situations!!!!!
        // if reset flag set, reset the buffer pool
        if (ResetBufferFlag) {
	    for (nodeNum = 0; nodeNum < NumNodes; nodeNum++) {
   	        e = Nodes[nodeNum]->bufMgr->ResetBuffer();
		if (e != OK) 
		    printf("Node %d, bufMgr->ResetBuffer returned error %d\n", 
			   nodeNum, e);
	    }
	}

        // Forks off a query scheduler process to hand this specific query.
        // Right now, just runs em all on this node.    Need to load bal this?

	reqMsg->query->transId = transCounter++;

	queryClass = reqMsg->query->jobClass;

        // update class and plan current MPLs
        reqMsg->query->jobClass->currentMPL++;
	((RootPlan*) (reqMsg->query))->randPlan->currentMPL++;

	// if the currentMPL of class already at maximum or currentMPL
	//    of plan at maximum, queue up the query in the job class 
        //    queue. queries are dequeued in a first-come first served 
	//    basis when a query completes
	if ((queryClass->readyMPL >= queryClass->maxMPL) ||
 	    (((RootPlan*) (reqMsg->query))->randPlan->readyMPL >=
	     ((RootPlan*) (reqMsg->query))->randPlan->maxMPL))
	{
            if (Flags[TRACE_SCHED])
	         printf("readyMPL = maxMPL query waits\n");
	    queryClass->mplQueue->enqueue(gSchedNode, reqMsg, NULL, NULL);
        }
	else
	    // start scheduling the query
	    startQuery(gSchedNode, reqMsg);
    }
    delete GlobalMemoryWaitQueue;
    delete GlobalMPLWaitQueue;
}





// start scheduling the query
void startQuery(Node* gSchedNode, QueryMsg* reqMsg)
{
    int e = OK;
    Schedule *schedule, *waitSchedule;


    schedule = NULL;
    waitSchedule = NULL;

    e = initialSchedule(reqMsg, schedule, waitSchedule);
    if (e == OK)
    {
        // update class and plan ready MPLs
        reqMsg->query->jobClass->readyMPL++;
	((RootPlan*) (reqMsg->query))->randPlan->readyMPL++;


        // if the global query queue is not empty add the query 
	// to the queue
	if ((GlobalMemoryWaitQueue->nonempty()) &&
	    (GlobalMemoryWaitQueue->checkWaiters(((RootPlan*) reqMsg->query)->totalMaxPagesNeeded)))
	{
	  
	     GlobalMemoryWaitQueue->enqueue(gSchedNode, reqMsg, 
					   schedule, waitSchedule);
	     if (Flags[TRACE_SCHED])
	       printf("Query Queue not empty : new query queued up\n");
	}
	else
	    if (CanRun((RootPlan*) (reqMsg->query))) {
	        // form the schedules. also does the resource allocation
		e = allocateResources(reqMsg, schedule, waitSchedule);
		
		if (e == OK)
		    // fork off a new query scheduler to handle the query
		    QueryScheduler(gSchedNode, reqMsg, schedule, waitSchedule);
		else
		    // report scheduling error to the user
		    printf("*ERROR*: in scheduling query. abort execution\n");
		  
	    }
	    else  // not enough memory
	    {
		GlobalMemoryWaitQueue->enqueue(gSchedNode, reqMsg, 
					       schedule, waitSchedule);
		if (Flags[TRACE_SCHED])
		    printf("Query cannot run now: queued up\n");
	    }
    }
    else // e == NOTOK
    {
        // destroy the old schedules
	DeleteSched(schedule);
	DeleteSched(waitSchedule);
    }

}




