class QueryPlan;

// structure of each split table entry
struct NetBufRec {
    short	numTuples;	// number of tuples on the network page
    NetBufRec* 	next;
    Mailbox*	destNode;	// mailbox of destination node
};

//////////////////////////////////////////////////////////////////////////////

class SplitTable {
  private:
    static int numSplitTables;
    static class SplitTable *freeSplitTable;
    SplitTable *next;
  public:
    int         splitCnt;                         
    NetBufRec*	splitArray;
    bool        fullBroadcast;      // if pages should be sent to everyone
    int	        tupleSize;	        // size of each tuple in bytes
    int         tupsPerNetPageOut;  // number of tuples per net page
    int         skewEntry;          // KLUDGE for skew; indicates entry to be 

     SplitTable(int, int, bool);
    ~SplitTable();
    inline void* operator new(size_t size);  // new is overloaded 
    inline void operator delete(void* oldRec);

    // put a tuple in the split table.  if the page is full it will be
    // sent out to appropriate node, or, if fullBroadcast to every node.
    void putTuples(Node* n, Mailbox* source, bool eofFlag, int numTuples);

    // send full buffer to its destination 
    void sendNetBuffer(int tableEntry, bool eofFlag, Node* n, Mailbox* source);
 
   // flush all split table buffers to their destinations
   void flushNetBufs(Node* n, Mailbox* source); 
};

//////////////////////////////////////////////////////////////////////////////

inline void* SplitTable::operator new(size_t size)
{
    SplitTable *newRec;

    if (freeSplitTable == NULL) {
        // allocate a new Rec
        newRec = (SplitTable *) new char[size];
	if (Flags[TRACE_LEAK])
	  printf("new SplitTable %d\n", ++numSplitTables);
      } else {
        // reuse an existing Rec
        newRec = freeSplitTable;
        freeSplitTable = freeSplitTable->next;
      }
    return newRec;
  };

//////////////////////////////////////////////////////////////////////////////

inline void SplitTable::operator delete(void* oldSplitTable)
{
    // put the Rec on the free list
    ((SplitTable *) oldSplitTable)->next = freeSplitTable;
    freeSplitTable = (SplitTable *) oldSplitTable;
  };

